import asyncio
import platform
import re
import subprocess
import socket
import pyautogui
import os
import pygetwindow as gw
from rich.console import Console
import pygetwindow as gw
from pywinauto import Application, Desktop
from worker_automate_hub.api.client import change_robot_status_true 
from worker_automate_hub.models.dto.rpa_historico_request_dto import RpaHistoricoStatusEnum, RpaRetornoProcessoDTO, RpaTagDTO, RpaTagEnum
from worker_automate_hub.models.dto.rpa_processo_rdp_dto import RpaProcessoRdpDTO
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.util import worker_sleep

console = Console()


class RDPConnection:
    def __init__(self, task: RpaProcessoRdpDTO):
        self.task = task
        self.ip = task.configEntrada.get("ip")
        self.user = task.configEntrada.get("user")
        self.password = task.configEntrada.get("password")
        self.processo = task.configEntrada.get("processo")
        self.uuid_robo = task.configEntrada.get("uuidRobo")
    
    async def verificar_conexao(self) -> bool:
        sistema_operacional = platform.system().lower()
        console.print(f"Sistema operacional detectado: {sistema_operacional}")
        if sistema_operacional == "windows":
            comando_ping = ["ping", "-n", "1", "-w", "1000", self.ip]
        else:
            comando_ping = ["ping", "-c", "1", "-W", "1", self.ip]
        console.print(f"Executando comando de ping: {' '.join(comando_ping)}")
        try:
            resposta_ping = subprocess.run(comando_ping, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
            ping_alcancado = resposta_ping.returncode == 0
            console.print(f"Ping {'sucesso' if ping_alcancado else 'falhou'}")
        except Exception as e:
            logger.error(f"Erro ao executar ping: {e}")
            ping_alcancado = False
        
        porta_aberta = False
        try:
            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
                sock.settimeout(10)
                console.print(f"Verificando porta 3389 em {self.ip}")
                resposta_porta = sock.connect_ex((self.ip, 3389))
                porta_aberta = resposta_porta == 0
                console.print(f"Porta 3389 {'aberta' if porta_aberta else 'fechada'}")
        except Exception as e:
            logger.error(f"Erro ao verificar a porta RDP: {e}")
            porta_aberta = False
        
        return ping_alcancado and porta_aberta
    
    async def clicar_no_titulo(self):
        janelas_rdp = [
            win
            for win in gw.getAllTitles()
            if self.ip in win
        ]

        for titulo in janelas_rdp:
            janela = gw.getWindowsWithTitle(titulo)[0]
            if not janela:
                console.print(f"Erro ao localizar a janela: {titulo}")
                continue

            console.print(f"Processando janela: {titulo}")
            x, y = janela.left, janela.top

            try:
                pyautogui.moveTo(x + 2, y + 2)
                pyautogui.hotkey("alt", "space")
                await worker_sleep(2)
                pyautogui.press("down", presses=6, interval=0.1)
                await worker_sleep(2)
                pyautogui.hotkey("enter")
                await worker_sleep(2)
                pyautogui.hotkey("enter")
                break

            except Exception as e:
                console.print(f"Erro ao interagir com a janela {titulo}: {e}")

    async def open_rdp(self):
        base_path = os.path.join("worker_automate_hub", "tasks", "jobs")
        rdp_file_path = os.path.join(base_path, "connection.rdp")
        os.makedirs(base_path, exist_ok=True)
        
        rdp_content = f"""screen mode id:i:1
            desktopwidth:i:1920
            desktopheight:i:1080
            use multimon:i:0
            session bpp:i:32
            compression:i:1
            keyboardhook:i:2
            displayconnectionbar:i:1
            disable wallpaper:i:0
            allow font smoothing:i:1
            allow desktop composition:i:1
            disable full window drag:i:0
            disable menu anims:i:0
            disable themes:i:0
            disable cursor setting:i:0
            bitmapcachepersistenable:i:1
            full address:s:{self.ip}
            username:s:{self.user}
            smart sizing:i:1
            winposstr:s:0,1,{(1920 // 4)},{(1080 // 4)},{(1920 // 4) + (1920 // 2)},{(1080 // 4) + (1080 // 2)}
        """

        try:
            with open(rdp_file_path, "w") as file:
                file.write(rdp_content)
            print(f"Arquivo RDP criado: {rdp_file_path}")
        except Exception as error:
            print(f"Erro ao montar o arquivo RDP: {error}")
            return

        try:
            subprocess.Popen(["mstsc", rdp_file_path], close_fds=True, start_new_session=True)
            console.print("Conexão RDP iniciada.")
        except Exception as error:
            console.print(f"Erro ao abrir a conexão RDP: {error}")

    async def verificar_conexao_estabelecida(self):
        janelas_rdp = [
            win
            for win in gw.getAllTitles()
            if self.ip in win
        ]

        janela = None

        for titulo in janelas_rdp:
            janela = gw.getWindowsWithTitle(titulo)[0]
            if not janela:
                console.print(f"Erro ao localizar a janela: {titulo}")
                continue
            else:
                return True

    async def conectar(self):
        console.print(f"Iniciando cliente RDP para {self.ip}")
        try:
            await worker_sleep(3)
            await self.open_rdp()
            await worker_sleep(5)
            
            console.print("Procurando janela RDP")
            try:
                possible_titles = ["Conexão de Área de Trabalho Remota", "Ligação ao ambiente de trabalho remoto"]
                app = None
                app_window = None

                for title in possible_titles:
                    try:
                        app = Application(backend="uia").connect(title=title)
                        app_window = app.window(title=title)
                        console.print(f"Janela encontrada: {title}")
                        break
                    except Exception:
                        continue

                if not app or not app_window:
                    raise Exception("Nenhuma janela com título correspondente foi encontrada.")

                app_window.set_focus()
                console.print("Janela RDP ativada.")

                pyautogui.press("left")
                await worker_sleep(1)
                pyautogui.hotkey("enter")
                await worker_sleep(3)

            except Exception as e:
                console.print(f"Erro ao ativar a janela RDP: {e}", style="bold red")

            app_window = None
            possible_titles_patterns = [
                re.compile(r"Conexão de Área de Trabalho Remota", re.IGNORECASE),
                re.compile(r"Ligação ao ambiente de trabalho remoto", re.IGNORECASE),
                re.compile(r"Segurança do Windows", re.IGNORECASE)
            ]

            janelas = Desktop(backend="uia").windows()
            for janela in janelas:
                titulo = janela.window_text()
                for pattern in possible_titles_patterns:
                    if pattern.search(titulo):
                        try:
                            app = Application(backend="uia").connect(title_re=pattern)
                            app_window = app.window(title_re=pattern)
                            console.print(f"Janela encontrada: {titulo}")
                            break
                        except Exception as e:
                            console.print(f"Erro ao conectar a janela '{titulo}': {e}")
                if app_window:
                    break

            await worker_sleep(2)

            if app_window:
                try:
                    app_window.restore()
                    app_window.set_focus()
                    console.print("Foco definido com sucesso.")
                except Exception as e:
                    console.print(f"Erro ao definir foco na janela: {e}")
            else:
                console.print("Nenhuma janela correspondente foi encontrada.")

            await worker_sleep(5)
            console.print("Inserindo senha.")
            pyautogui.write(self.password, interval=0.1)
            pyautogui.press("enter")
            await worker_sleep(5)
            pyautogui.press("left")
            await worker_sleep(5)
            pyautogui.press("enter")
            console.print("Credenciais inseridas.")
            await worker_sleep(5)
            
            try:
                base_path = os.path.join("worker_automate_hub", "tasks", "jobs")
                rdp_file_path = os.path.join(base_path, "connection.rdp")
                if os.path.exists(rdp_file_path):
                    os.remove(rdp_file_path)
            except Exception as error:
                console.print(f"Erro ao deletar arquivo 'connection.rdp'. Error: {error}")

            conexao_rdp_aberta = await self.verificar_conexao_estabelecida()
            if conexao_rdp_aberta is not True:
                raise ConnectionError("Não foi possível estabelecer conexão RDP.")
            
            return True

        except Exception as e:
            logger.error(f"Erro ao tentar conectar via RDP: {e}")
            return False

async def conexao_rdp(task: RpaProcessoRdpDTO) -> RpaRetornoProcessoDTO:
    try:
        console.print("Iniciando processo de conexão RDP.")
        rdp = RDPConnection(task)
        conectado = await rdp.verificar_conexao()
        if not conectado:
            logger.warning("Não foi possível estabelecer conexão RDP. Verifique o IP e a disponibilidade da porta.")
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno="Não foi possível estabelecer conexão RDP. Verifique o IP e a disponibilidade da porta.",
                status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

        sucesso_conexao = await rdp.conectar()
        if sucesso_conexao:
            console.print("Processo de conexão RDP executado com sucesso.")
            
            #await rdp.clicar_no_titulo()

            #Altera coluna ativo para true or false 
            await change_robot_status_true(rdp.uuid_robo)

            return RpaRetornoProcessoDTO(
                sucesso=True,
                retorno="Conexão RDP estabelecida com sucesso.",
                status=RpaHistoricoStatusEnum.Sucesso,
            )
        else:
            logger.error("Falha ao tentar conectar via RDP.")
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno="Falha ao tentar conectar via RDP.",
                status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )
    except Exception as ex:
        err_msg = f"Erro ao executar conexão RDP: {ex}"
        logger.error(err_msg)
        console.print(err_msg, style="bold red")
        return RpaRetornoProcessoDTO(
            sucesso=False,
            retorno=err_msg,
            status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
        )



import os
import subprocess
import sys
import threading
from pathlib import Path
from typing import Optional

import psutil
import requests
import toml
from packaging import version
from rich.console import Console

from worker_automate_hub.utils.logger import logger

console = Console()


def update_version_in_toml(file_path: Path, new_version: str):
    """
    Atualiza a versão do worker no arquivo TOML.

    Args:
        file_path (Path): Caminho completo para o arquivo TOML.
        new_version (str): Nova versão para ser escrita no arquivo TOML.
    """
    try:
        # Abrir e carregar o arquivo TOML
        with open(file_path, "r") as file:
            config = toml.load(file)

        # Alterar a versão
        config["params"]["version"] = new_version

        # Salvar as alterações de volta no arquivo TOML
        with open(file_path, "w") as file:
            toml.dump(config, file)

        console.print(f"Versão atualizada para {new_version} no arquivo {file_path}")
    except Exception as e:
        console.print(f"Erro ao atualizar a versão: {e}")

def create_update_script(new_version: str):
    """
    Cria um arquivo de script .bat para atualizar a versão do worker com o pipx e reiniciar a aplicação.

    Args:
        new_version (str): Nova versão para ser escrita no arquivo TOML.

    Returns:
        Path: Caminho completo para o arquivo update.bat criado.
    """
    try:
        current_dir = Path.cwd()

        # Caminho do arquivo TOML
        toml_file_path = os.path.join(current_dir, 'settings.toml')        

        # Comando para atualizar o pacote via pipx
        update_command = 'pipx upgrade worker-automate-hub --force'

        # Comando para atualizar a versão no arquivo TOML
        update_toml_command = f'python -c "import toml; config=toml.load(\'{toml_file_path}\'); config[\'params\'][\'version\'] = \'{new_version}\'; toml.dump(config, open(\'{toml_file_path}\', \'w\'))"'

        # Comando para reiniciar a aplicação
        restart_command = f'start cmd /K "cd /D {current_dir} && worker-startup.bat"'

        # Caminho do arquivo de log
        log_file_path = os.path.join(current_dir, 'update.log')

        # Criando o conteúdo do script .bat
        script_content = f"""
        @echo off
        taskkill /IM worker.exe /F
        timeout /t 10 /nobreak
        {update_command}
        if %errorlevel% equ 0 (
            echo Update successful, updating version in TOML file...
            {update_toml_command}
            echo Version updated successfully, restarting the application...
            {restart_command}
        ) else (
            echo Update failed. > "{log_file_path}"
            echo Error code: %errorlevel% >> "{log_file_path}"
            echo See pipx output for more details. >> "{log_file_path}"
        )
        exit
        """

        # Caminho completo para o arquivo update.bat
        bat_file_path = os.path.join(current_dir, 'update.bat')

        # Escrevendo o conteúdo no arquivo update.bat
        with open(bat_file_path, 'w') as file:
            file.write(script_content.strip())

        log_msg = "Arquivo de update criado com sucesso!"
        logger.info(log_msg)
        console.print(f"\n{log_msg}\n", style="bold green")
        
        return bat_file_path

    except Exception as e:
        err_msg = "Erro ao criar o arquivo de update: {e}"
        logger.error(err_msg)
        console.print(f"\n{err_msg}\n", style="bold red")


def run_update_script(
    bat_file_path: str,
    stop_event: threading.Event
) -> None:
    """
    Executa o script .bat para atualizar o pacote.

    Args:
    - bat_file_path (str): Caminho completo para o arquivo .bat.
    - stop_event (threading.Event): Um objeto Event que é usado para
      sinalizar para as threads que devem parar.

    Returns:
    None
    """
    try:
        # Executando o script .bat
        subprocess.Popen(['start', 'cmd', '/c', bat_file_path], shell=True)
        
        log_msg = "update.bat executado com sucesso!"
        console.print(f"\n{log_msg}\n", style="bold green")
        logger.info(log_msg)        

        # Sinalizar para as threads que devem parar
        stop_event.set()

        # Encerrar o programa atual
        os.system('exit')
        sys.exit(0)        

    except subprocess.CalledProcessError as e:
        err_msg = f"Erro ao executar o update.bat: {e}"
        console.print(f"\n{err_msg}\n", style="bold red")
        logger.error(f"{err_msg}")

def get_installed_version(package_name: str) -> Optional[str]:
    
    """
    Retorna a versão atual do pacote instalada.

    Args:
    - package_name (str): Nome do pacote que se deseja obter a versão.

    Returns:
    - str: A versão atual do pacote ou None se o pacote não for encontrado.
    """
    try:
        result = subprocess.run(['pipx', 'list'], capture_output=True, text=True, check=True)
        lines = result.stdout.splitlines()
        for line in lines:
            if package_name in line:
                # Parse the version from the line, assuming the line format is 'package_name <version>'
                return line.split(' ')[5].replace(',', '')
    except subprocess.CalledProcessError as e:
        err_msg = f"Ocorreu um erro ao obter a versão do pacote instalada: {e}"
        console.print(f"\n{err_msg}\n", style="bold red")
        logger.error(err_msg)
    return None

def check_for_update(stop_event: threading.Event):
    """
    Verifica se há uma nova versão do pacote disponível no PyPI.

    Se houver uma versão mais recente disponível, atualiza o pacote via pipx e
    reinicia a aplicação.

    Args:
        stop_event (threading.Event): Um objeto Event que é usado para
            sinalizar para as threads que devem parar.
    """
    try:
        package_name = "worker-automate-hub"
        current_version = get_installed_version(package_name)

        response = requests.get(f"https://pypi.org/pypi/{package_name}/json")
        latest_version = response.json()
        console.print("last: "+latest_version["info"]["version"])
        console.print("current: "+current_version)
        if version.parse(latest_version["info"]["version"]) > version.parse(current_version):
            console.print(
                f"Uma nova versão [bold cyan]({latest_version["info"]["version"]})[/bold cyan] está disponível. Atualizando..."
            )
            bat_file_path = create_update_script(latest_version["info"]["version"])
            run_update_script(bat_file_path, stop_event)
        else:            
            current_dir = Path.cwd()

            # Caminho do arquivo TOML
            toml_file_path = os.path.join(current_dir, 'settings.toml')
            update_version_in_toml(toml_file_path, current_version)
            console.print("\nVocê está usando a versão mais atualizada.\n", style="green")
    except Exception as e:
        err_msg = f"Erro ao verificar novas versões do pacote: {e}"
        console.print(f"\n{err_msg}\n", style="bold red")
        logger.error(err_msg)

def close_other_cmd_windows():
    """
    Fecha outras janelas do cmd, deixando apenas a janela atual aberta.
    """
    try:
        current_pid = os.getpid() 
        for proc in psutil.process_iter(attrs=["pid", "name", "cmdline"]):
            try:           
                if proc.info["name"] == "cmd.exe" and proc.info["pid"] != current_pid:
                    proc.terminate()
            except (psutil.NoSuchProcess, psutil.AccessDenied):
                continue

    except Exception as e:
        err_msg = f"Erro ao fechar outras janelas do cmd: {e}"
        console.print(f"\n{err_msg}\n", style="bold red")
        logger.error(err_msg)



import aiohttp
from rich.console import Console

from worker_automate_hub.config.settings import load_env_config
from worker_automate_hub.utils.logger import logger

console = Console()


async def burn_queue(id_fila: str):
    env_config, _ = load_env_config()
    try:

        headers_basic = {"Authorization": f"Basic {env_config["API_AUTHORIZATION"]}"}

        async with aiohttp.ClientSession(
            connector=aiohttp.TCPConnector(verify_ssl=True)
        ) as session:
            async with session.delete(
                f"{env_config["API_BASE_URL"]}/fila/burn-queue/{id_fila}",
                headers=headers_basic,
            ) as response:
                if response.status == 200:
                    logger.info("Fila excluida com sucesso.")
                    console.print("\nFila excluida com sucesso.\n", style="bold green")
                else:
                    logger.error(f"Erro ao excluir a fila: {response.content}")
                    console.print(
                        f"Erro ao excluir a fila: {response.content}", style="bold red"
                    )

    except Exception as e:
        err_msg = f"Erro remover registro da fila: {e}"
        logger.error(err_msg)
        console.print(
            f"{err_msg}\n",
            style="bold red",
        )
        return None


async def unlock_queue(id: str):
    env_config, _ = load_env_config()
    try:
        headers_basic = {"Authorization": f"Basic {env_config["API_AUTHORIZATION"]}"}

        async with aiohttp.ClientSession(
            connector=aiohttp.TCPConnector(verify_ssl=True)
        ) as session:
            async with session.get(
                f"{env_config["API_BASE_URL"]}/fila/unlock-queue/{id}",
                headers=headers_basic,
            ) as response:
                return await response.text()

    except Exception as e:
        err_msg = f"Erro ao desbloquear a fila: {e}"
        logger.error(err_msg)
        console.print(
            f"{err_msg}\n",
            style="bold red",
        )
        return None



import asyncio
import io
import os
import re
import shutil
import warnings
from datetime import datetime, timedelta


import pyperclip
import pyautogui
from pywinauto.application import Application
from rich.console import Console
from dateutil.relativedelta import relativedelta

from worker_automate_hub.models.dto.rpa_historico_request_dto import RpaHistoricoStatusEnum, RpaRetornoProcessoDTO, RpaTagDTO, RpaTagEnum
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import RpaProcessoEntradaDTO
from worker_automate_hub.utils.get_creds_gworkspace import GetCredsGworkspace
from googleapiclient.discovery import build
from worker_automate_hub.utils.logger import logger
from pywinauto_recorder import set_combobox
from worker_automate_hub.api.client import get_config_by_name, get_valor_remessa_cobranca, send_file, sync_get_config_by_name
from worker_automate_hub.utils.util import (
    create_temp_folder,
    delete_folder,
    kill_all_emsys,
    kill_process, 
    login_emsys,
    save_pdf_emsys, 
    type_text_into_field,
    worker_sleep,
    set_variable,
    )

pyautogui.PAUSE = 0.5
ASSETS_BASE_PATH = 'assets/fidc/'
PDF_SUFFIX = "_PDF.pdf"
console = Console()



async def remessa_cobranca_cnab240(task: RpaProcessoEntradaDTO) -> RpaRetornoProcessoDTO:
    '''
       Processo FIDC - Remessa de Cobrança CNAB240
    '''
    try:
        #Setando tempo de timeout
        multiplicador_timeout = int(float(task.sistemas[0].timeout))
        set_variable("timeout_multiplicador", multiplicador_timeout)
        
        #Pegando nome do usuario
        nome_usuario = os.environ.get('USERNAME') or os.environ.get('USER')
        nome_pasta = f"{nome_usuario}_arquivos"

        #Delete temp folder
        await delete_folder(nome_pasta)
        #Cria Pasta temporaria
        temp_folder = await create_temp_folder()
        temp_folder = temp_folder +'\\'
        #Pega Config para logar no Emsys
        config = await get_config_by_name("login_emsys")
        folders_paths = await get_config_by_name("Folders_Fidc")

        #Abre um novo emsys
        await kill_all_emsys()
        app = Application(backend='win32').start("C:\\Rezende\\EMSys3\\EMSys3.exe")
        warnings.filterwarnings("ignore", category=UserWarning, message="32-bit application should be automated using 32-bit Python")
        console.print("\nEMSys iniciando...", style="bold green")
        return_login = await login_emsys(config.conConfiguracao, app, task)

        if return_login.sucesso == True:
            type_text_into_field('Remessa de Cobranca', app['TFrmMenuPrincipal']['Edit'], True, '50')
            pyautogui.press('enter')
            await worker_sleep(1)
            pyautogui.press('enter')
            console.print(f"\nPesquisa: 'Retorno de Cobranca' realizada com sucesso", style="bold green")
        else:
            logger.info(f"\nError Message: {return_login.retorno}")
            console.print(f"\nError Message: {return_login.retorno}", style="bold red")
            return return_login
        
        await worker_sleep(10)
        
        # Identificando jenela principal
        app = Application().connect(title="Gera Arquivo Cobranca", backend="uia")
        main_window_arquivo_cobranca = app["Gera Arquivo Cobranca"]
        main_window_arquivo_cobranca.set_focus()

        # Digitando Cobrança
        cobranca = main_window_arquivo_cobranca.child_window(class_name="TDBIEditCode", found_index=2)
        console.print("Selecionando Cobrança", style='bold green')
        cobranca.type_keys("4")
        pyautogui.hotkey("tab") 
        await worker_sleep(5)
        pyautogui.press("down", presses=3, interval=0.5)
        await worker_sleep(2)
        pyautogui.hotkey("enter")

        await worker_sleep(10)
        
        # Identificando jenela principal
        app = Application().connect(title="Gera Arquivo Cobranca", backend="uia")
        main_window_arquivo_cobranca = app["Gera Arquivo Cobranca"]

        # Selecionando caminho do arquivo
        field_arquivo = main_window_arquivo_cobranca.child_window(class_name="TDBIEditString", found_index=0)
        text_field_arquivo = field_arquivo.window_text()
        new_text_field_arquivo = str(re.search(r'REM(.*)\.txt', text_field_arquivo).group(1))
        new_text_field_arquivo = 'R00102#####.001'.replace('#####', str(new_text_field_arquivo).zfill(5))
        field_arquivo.set_focus()
        
        field_arquivo.double_click_input()
        field_arquivo.set_edit_text("")
        # field_arquivo.type_keys(folders_paths.conConfiguracao['remessa_cobranca_path'] + new_text_field_arquivo, with_spaces=True)
        field_arquivo.type_keys(temp_folder + new_text_field_arquivo, with_spaces=True)

        await worker_sleep(2)   
        #Seleciona Banco
        pyautogui.click(810, 397)
        pyautogui.press("down", presses=2)

        pyautogui.hotkey("enter")
        
        # Data atual
        data_atual = datetime.now()

        # Data(8 dias atrás)
        start_date = data_atual - timedelta(days=8)
        # Data(1 dia atrás)
        end_date = data_atual - timedelta(days=1)


        #Data de emissão
        pyautogui.click(700, 482)
        pyautogui.write(start_date.strftime("%d%m%Y"))
        pyautogui.click(780, 485)
        pyautogui.write(end_date.strftime("%d%m%Y"))

        #Data Vencimento
        pyautogui.click(900, 485)
        pyautogui.write(datetime.now().strftime("%d%m%Y"))
        # pyautogui.write(start_date)
        pyautogui.click(1000, 485)
        pyautogui.write((datetime.now() + relativedelta(months=6)).strftime("%d%m%Y"))
        # pyautogui.write(end_date)

        filtro = main_window_arquivo_cobranca.child_window(class_name="TGroupBox", found_index=2)
        faturados_negociados = filtro.child_window(title="Faturados/Negociados", class_name="TRadioButton")
        faturados_negociados.click()
        somente_nosso_numero = main_window_arquivo_cobranca.child_window(title="Somente Títulos com Nosso Número",class_name="TCheckBox", found_index=0)
        somente_nosso_numero.click()

        # Clica Pesquisar Titulos
        pyautogui.click(1160, 548)

        await worker_sleep(10)

        #Clicando em sim para titulos
        pyautogui.click(920, 560)

        await worker_sleep(20)

        #Selecionando todas empresas na tela  Seleção de Empresa
        pyautogui.click(720, 620)

        #clcica em OK
        pyautogui.click(1100, 660)

        await worker_sleep(120)

        try:
            app = Application().connect(title="Informação", backend="uia")
            main_window_arquivo_cobranca = app["Informação"]
            if main_window_arquivo_cobranca.exists():
                return RpaRetornoProcessoDTO(
            sucesso=False,
            retorno="Nenhum titulo encontrado para a remessa de cobrança!",
            status=RpaHistoricoStatusEnum.Sucesso,
        )
        except Exception as ex:
            ...
        # Seleciona todos titulos
        pyautogui.click(1275, 600)
        # pyautogui.click(636, 591)
        
        await worker_sleep(10)

        # Pegando Total do Emsys
        app = Application().connect(title="Gera Arquivo Cobranca", backend="uia")
        main_window_arquivo_cobranca = app["Gera Arquivo Cobranca"]
        main_window_arquivo_cobranca.set_focus()
        field_total_emsys = main_window_arquivo_cobranca.child_window(class_name="TDBIEditNumber", found_index=3).window_text()
        #Pegando total do banco
        # total_db = await get_valor_remessa_cobranca(data_atual.strftime("%Y-%m-%d"))
        total_db = await get_valor_remessa_cobranca("2025-01-29")
        console.print("Total DB: " + str(total_db))
        console.print("Total EMSYS: " + str(field_total_emsys))
        field_total_emsys = 0
        total_db = 0
        if total_db == field_total_emsys:
            #Clica gerar cobrança
            await worker_sleep(5)
            pyautogui.click(1135, 789)
        else:
            log_msg = "Valores divergem! \nValor no EmSys: " + str(field_total_emsys) + " \nValor dos titulos: " + str(total_db)
            return RpaRetornoProcessoDTO(sucesso=False, 
                                         retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, 
                                         tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)])    

        await worker_sleep(10)

        # Confirma geração com sucesso
        max_try = 10
        current_try = 0
        while current_try <= max_try:
            try:
                app = Application().connect(title="Information", class_name="TMessageForm")
                window_cobranca = app["Information"]
                if window_cobranca.exists():
                    #Click OK
                    window_cobranca.set_focus()
                    await worker_sleep(1)
                    pyautogui.click(958, 560)
                    break
            except Exception as ex:
                log_msg = f"Erro ao encontrar janela de confirmação de cobrança: {str(ex)}"
                console.print(log_msg, style="bold red")
                
            current_try += 1
            await worker_sleep(20)
            
        if current_try >= max_try:
            return RpaRetornoProcessoDTO(
                sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])    

        # # Substituindo arquivo original e fazedno upload (feito logo apos gerar para  garaintir que o arquivo não se perca)
        # origem = folders_paths.conConfiguracao['remessa_cobranca_path'] + new_text_field_arquivo
        origem = temp_folder + new_text_field_arquivo
        
        # Abrindo o arquivo original para leitura
        file = open(origem, 'r')
        file_text = file.read()
        file.close()
        # Realizando as substituições
        file_text = file_text.replace(
        '074737350001813576383             0316820000000058114 SIM REDE DE POSTOS LTDA',
        '45931917000148003576383001417019  0316820000000058114 ARGENTA FUNDO DE INVEST').replace(
        '20074737350001813576383             0316820000000058114 SIM REDE DE POSTOS LTDA',
        '2045931917000148003576383001417019  0316820000000058114 ARGENTA FUNDO DE INVEST')
        # Sobrescrevendo o arquivo original com o conteúdo alterado
        file = open(origem, 'w')
        file.write(file_text)
        file.close()

        console.print(f"Substituições realizadas com sucesso no arquivo original: {origem}")
        try:
            with open(origem, 'rb') as file:
                file_bytes = io.BytesIO(file.read())
            # Enviando o arquivo 001 para BOF
            await send_file(task.historico_id, new_text_field_arquivo, "001", file_bytes, file_extension="001")
        except Exception as e:
            log_msg=(f"Erro ao enviar o arquivo 001: {e}")
            console.print(log_msg, style="bold red")
            return RpaRetornoProcessoDTO(
            sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])


        # Clica em 'Yes' imprimir listagem
        await worker_sleep(10)
        try:
            app = Application().connect(title="Confirm", class_name="TMessageForm")
            window_listagem_alfabetica = app["Confirm"]
            if window_listagem_alfabetica.exists():
                window_listagem_alfabetica.set_focus()
                yes_btn = window_listagem_alfabetica.child_window(title="&Yes", class_name="TButton")
                yes_btn.click()
        except Exception as ex:
            log_msg = f"Erro ao encontrar janela de impressão de listagem: {str(ex)}"
            console.print(log_msg, style="bold red")
            return RpaRetornoProcessoDTO(
            sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])
        
        #Clica em "Yes" ordem alfabética
        await worker_sleep(10)
        try:
            app = Application().connect(title="Confirm", class_name="TMessageForm")
            window_listagem_alfabetica = app["Confirm"]
            if window_listagem_alfabetica.exists():
                window_listagem_alfabetica.set_focus()
                yes_btn = window_listagem_alfabetica.child_window(title="&Yes", class_name="TButton")    
                yes_btn.click()  
        except Exception as ex:
            log_msg = f"Erro ao encontrar janela de impressão de listagem em ordem alfabética: {str(ex)}"
            console.print(log_msg, style="bold red")            
            return RpaRetornoProcessoDTO(
            sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])

        await worker_sleep(10)
        
        # await save_pdf_emsys(folders_paths.conConfiguracao['remessa_cobranca_path'] + new_text_field_arquivo +" (PDF)")
        await save_pdf_emsys(temp_folder + new_text_field_arquivo + PDF_SUFFIX)
        
        await worker_sleep(5) 

        #Clica para não imprimir os boletos
        try:
            app = Application().connect(title="Confirm", class_name="TMessageForm")
            window_listagem_alfabetica = app["Confirm"]
            if window_listagem_alfabetica.exists():
                window_listagem_alfabetica.set_focus()
                pyautogui.click(998, 562)
        except Exception as ex:
            log_msg = f"Erro ao encontrar janela 'deseja imprimir os boletos?': {str(ex)}"
            console.print(log_msg, style="bold red")            
            return RpaRetornoProcessoDTO(
            sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])
                
        try:
            # Enviando arquivo PDF para BOF
            with open(temp_folder + new_text_field_arquivo + PDF_SUFFIX, 'rb') as file:
                file_bytes_pdf = io.BytesIO(file.read())
            await send_file(task.historico_id, new_text_field_arquivo, "pdf", file_bytes_pdf, file_extension="pdf")
        except Exception as e:
            log_msg=(f"Erro ao enviar o arquivo PDF: {e}")
            console.print(log_msg, style="bold red")
            return RpaRetornoProcessoDTO(
            sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])
        
        try:
            #Mover para pasta correta
            shutil.move(origem, folders_paths.conConfiguracao["remessa_cobranca_path_prod"])
        except Exception as e:
            log_msg=(f"Erro ao mover o arquivo para pasta de remessa: {e}")
            console.print(log_msg, style="bold red")
            return RpaRetornoProcessoDTO(
            sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])
    
        #Delete temp folder
        await delete_folder(nome_pasta)
        return RpaRetornoProcessoDTO(
            sucesso=True,
            retorno="Processo Remessa de Cobranca CNAB240 concluido com sucesso",
            status=RpaHistoricoStatusEnum.Sucesso,
        )

    except Exception as ex:
        log_msg = f"Erro Processo Remessa de Cobranca CNAB240: {str(ex)}"
        console.print(log_msg, style="bold red")
        return RpaRetornoProcessoDTO(
        sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])


import getpass
import warnings
import os
import io
import uuid
from datetime import datetime, date
import calendar

import pyautogui
import pyperclip
import pytesseract
from pywinauto.application import Application
from pywinauto.mouse import double_click
from pywinauto.keyboard import send_keys
import win32clipboard
from PIL import Image, ImageEnhance
from rich.console import Console

from worker_automate_hub.api.client import (
    get_config_by_name,
    send_file,
)

from worker_automate_hub.models.dto.rpa_historico_request_dto import (
    RpaHistoricoStatusEnum,
    RpaRetornoProcessoDTO,
    RpaTagDTO,
    RpaTagEnum,
)
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import (
    RpaProcessoEntradaDTO,
)
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.util import (
    is_window_open,
    is_window_open_by_class,
    login_emsys_fiscal,
    set_variable,
    type_text_into_field,
    worker_sleep,
    kill_all_emsys,
)

pyautogui.PAUSE = 0.5
pyautogui.FAILSAFE = False
console = Console()


async def sped_fiscal(task: RpaProcessoEntradaDTO) -> RpaRetornoProcessoDTO:
    """
    Processo que realiza as atividades de Sped no ERP EMSys(Linx) Fiscal.

    """
    try:
        # Get config from BOF
        console.print(task)
        try:
            get_config_gerar_inventario = await get_config_by_name(
                "SPED_gerar_inventario"
            )
            console.print(get_config_gerar_inventario)
            con_config = get_config_gerar_inventario.conConfiguracao
            if con_config is not None:
                gerar_inventario = con_config.get("gerar")
            else:
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno="Não foi possivel recuperar o valor da configuração Gerar Inventario, não sendo possivel seguir com o processo.",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
                )
        except Exception as e:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Não foi possivel recuperar o valor da configuração Gerar Inventario, erro: {e}.",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
            )

        config = await get_config_by_name("login_emsys_fiscal")
        # Seta config entrada na var sped_processar para melhor entendimento
        sped_processar = task.configEntrada
        multiplicador_timeout = int(float(task.sistemas[0].timeout))
        set_variable("timeout_multiplicador", multiplicador_timeout)

        historico_id = task.historico_id
        if historico_id:
            console.print("Historico ID recuperado com sucesso...\n")
        else:
            console.print(
                "Não foi possivel recuperar o histórico do ID, não sendo possivel enviar o arquivo SPED como retorno...\n"
            )
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno="Não foi possivel recuperar o histórico do ID, não sendo possivel enviar o arquivo SPED como retorno",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
            )

        # Fecha a instancia do emsys - caso esteja aberta
        await kill_all_emsys()

        app = Application(backend="win32").start("C:\\Rezende\\EMSys3\\EMSysFiscal.exe")
        warnings.filterwarnings(
            "ignore",
            category=UserWarning,
            message="32-bit application should be automated using 32-bit Python",
        )

        await worker_sleep(4)

        try:
            app = Application(backend="win32").connect(
                class_name="TFrmLoginModulo", timeout=50
            )
        except:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno="Erro ao abrir o EMSys Fiscal, tela de login não encontrada",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
            )

        return_login = await login_emsys_fiscal(config.conConfiguracao, app, task)
        if return_login.sucesso == True:
            type_text_into_field(
                "Sped Fiscal", app["TFrmMenuPrincipal"]["Edit"], True, "50"
            )
            pyautogui.press("enter")
            await worker_sleep(2)
            pyautogui.press("enter")
            console.print(
                f"\nPesquisa: 'Sped Fiscal' realizada com sucesso",
                style="bold green",
            )
        else:
            logger.info(f"\nError Message: {return_login.retorno}")
            console.print(f"\nError Message: {return_login.retorno}", style="bold red")
            return return_login

        await worker_sleep(6)
        console.print(
            "Verificando se a janela para gerar Sped foi aberta com sucesso...\n"
        )
        max_attempts = 15
        i = 0
        while i < max_attempts:
            gerar_sped = await is_window_open_by_class(
                "TFrmGerarSpeed", "TFrmGerarSpeed"
            )
            if gerar_sped["IsOpened"] == True:
                console.print("janela para gerar Sped foi aberta com sucesso...\n")
                break
            else:
                await worker_sleep(1)
                i = i + 1

        if i >= max_attempts:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno="Erro ao abrir a janela para gerar o Sped Fiscal, tela não encontrada",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
            )

        filial_cod = sped_processar.get("empresa")
        periodo_dt = sped_processar.get("periodo")

        # DIRETORIO TEMPORARIO ANTES DE SER MOVIDO PARA O GOOGLE DRIVE
        username = getpass.getuser()
        short_uuid = str(uuid.uuid4()).replace("-", "")[:6]
        temp_path_to_txt = (
            f"C:\\Users\\{username}\\Downloads\\{short_uuid}_{filial_cod}.txt"
        )
        console.print(f"Diretório temporario: {temp_path_to_txt}...\n")

        console.print("Navegando pela Janela de Gerar Sped Fiscal...\n")
        try:
            app = Application().connect(class_name="TFrmGerarSpeed")
            main_window = app["TFrmGerarSpeed"]

            main_window.set_focus()
            console.print(
                "Preenchendo as informações necessarias para gerar o sped fiscal... \n"
            )
            panel_TPage = main_window.child_window(class_name="TcxPageControl")
            panel_TTabSheet = panel_TPage.child_window(class_name="TcxTabSheet")
            panel_TTabSheet.wait("visible")

            console.print("Inserindo o diretório temporario para geração do sped...\n")
            filename = panel_TTabSheet.child_window(class_name="TFilenameEdit")
            filename.set_edit_text(temp_path_to_txt)
            await worker_sleep(2)

            console.print("Inserindo o período para geração do sped...\n")
            periodo = main_window.child_window(class_name="TDBIEditDate", found_index=0)
            periodo.set_edit_text(periodo_dt)
            await worker_sleep(1)
            periodo.type_keys("{TAB}")
            await worker_sleep(2)

            console.print("Selecionando a opção Gerar E115...\n")
            checkbox_gerar_e115 = main_window.child_window(
                class_name="TDBICheckBox", found_index=8
            )
            if not checkbox_gerar_e115.is_checked():
                checkbox_gerar_e115.click()
                console.print("Selecionado com sucesso... \n")
                await worker_sleep(2)

            console.print(
                "Selecionando a opção Gerar valores zerados para notas de cfop 5929 e 6929...\n"
            )
            checkbox_gerar_cfop = main_window.child_window(
                class_name="TCheckBox", found_index=8
            )
            if not checkbox_gerar_cfop.is_checked():
                checkbox_gerar_cfop.click()
                console.print("Selecionado com sucesso... \n")
                await worker_sleep(2)

            console.print("Selecionando a opção Gerar valores zerados para IPI...\n")
            checkbox_valores_ipi = main_window.child_window(
                class_name="TCheckBox", found_index=6
            )
            if not checkbox_valores_ipi.is_checked():
                checkbox_valores_ipi.click()
                console.print("Selecionado com sucesso... \n")
                await worker_sleep(2)

            console.print("Gerar valores zerados para Base ST e Icms ST...\n")
            checkbox_gerar_base_ct_icms_st = main_window.child_window(
                class_name="TCheckBox", found_index=7
            )
            if not checkbox_gerar_base_ct_icms_st.is_checked():
                checkbox_gerar_base_ct_icms_st.click()
                console.print("Selecionado com sucesso... \n")
                await worker_sleep(2)

            console.print("Nao gerar registro C176...\n")
            checkbox_n_regrar_registro_c176 = main_window.child_window(
                class_name="TCheckBox", found_index=1
            )
            if not checkbox_n_regrar_registro_c176.is_checked():
                checkbox_n_regrar_registro_c176.click()
                console.print("Selecionado com sucesso... \n")
                await worker_sleep(2)

            console.print("Nao gerar registro C1601...\n")
            checkbox_n_regrar_registro_c1601 = main_window.child_window(
                class_name="TDBICheckBox", found_index=2
            )
            if checkbox_n_regrar_registro_c1601.is_checked():
                checkbox_n_regrar_registro_c1601.click()
                console.print("Selecionado com sucesso... \n")
                await worker_sleep(2)

            console.print(
                "Todos os campos selecionados com sucesso, seguindo para geração do Sped Fiscal...\n"
            )
        except Exception as e:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Erro ao interagir com a tela de gerar sped fiscal, erro: {e}",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
            )

        ano_atual = datetime.now().year
        if str(periodo_dt) == f"02/{ano_atual}":
            console.print("Filtrando período pelo Inventario...\n")
            ano_passado = ano_atual - 1
            data_busca = f"31/12/{ano_passado}"

            periodo_filtro_inicio = main_window.child_window(
                class_name="TEditDate", found_index=1
            )
            periodo_filtro_inicio.set_edit_text(data_busca)
            await worker_sleep(2)
            periodo_filtro_fim = main_window.child_window(
                class_name="TEditDate", found_index=0
            )
            periodo_filtro_fim.set_edit_text(data_busca)
            await worker_sleep(2)
            periodo_btn_buscar = main_window.child_window(
                class_name="TBitBtn", found_index=0
            )
            periodo_btn_buscar.click()
            await worker_sleep(2)
            periodo_btn_buscar = main_window.child_window(
                class_name="TcxGrid", found_index=0
            )
            rect = periodo_btn_buscar.rectangle()
            center_x = (rect.left + rect.right) // 2
            center_y = (rect.top + rect.bottom) // 2
            double_click(coords=(center_x, center_y))
        elif gerar_inventario:
            mes_atual = datetime.now().month
            if mes_atual == 1:
                mes_anterior = 12
                ano_anterior = ano_atual - 1
            else:
                mes_anterior = mes_atual - 1
                ano_anterior = ano_atual

            ultimo_dia = calendar.monthrange(ano_anterior, mes_anterior)[1]
            data_ultimo_dia = date(ano_anterior, mes_anterior, ultimo_dia)
            data_formatada = data_ultimo_dia.strftime("%d/%m/%Y")

            periodo_filtro_inicio = main_window.child_window(
                class_name="TEditDate", found_index=1
            )
            periodo_filtro_inicio.set_edit_text(data_formatada)
            await worker_sleep(2)
            periodo_filtro_fim = main_window.child_window(
                class_name="TEditDate", found_index=0
            )
            periodo_filtro_fim.set_edit_text(data_formatada)
            await worker_sleep(4)
            periodo_btn_buscar = main_window.child_window(
                class_name="TBitBtn", found_index=0
            )
            periodo_btn_buscar.click()
            await worker_sleep(2)
            try:
                win32clipboard.OpenClipboard()
                win32clipboard.EmptyClipboard()
                win32clipboard.SetClipboardText("")
                win32clipboard.CloseClipboard()
            except:
                pyperclip.copy("")

            grid_inventario = main_window.child_window(
                class_name="TcxGrid", found_index=0
            )
            rect = grid_inventario.rectangle()
            center_x = (rect.left + rect.right) // 2
            center_y = (rect.top + rect.bottom) // 2
            pyautogui.moveTo(x=center_x, y=center_y)
            await worker_sleep(2)
            pyautogui.click()
            await worker_sleep(2)
            send_keys("^({HOME})")
            await worker_sleep(3)

            while True:
                last_line_inventario_emsys_fiscal = "x"
                await worker_sleep(1)
                with pyautogui.hold("ctrl"):
                    pyautogui.press("c")

                await worker_sleep(1)

                with pyautogui.hold("ctrl"):
                    pyautogui.press("c")

                win32clipboard.OpenClipboard()
                line_inventario_emsys_fiscal = win32clipboard.GetClipboardData().strip()
                win32clipboard.CloseClipboard()
                console.print(
                    f"Linha atual copiada do Emsys Fiscal: {line_inventario_emsys_fiscal}\nUltima Linha copiada: {last_line_inventario_emsys_fiscal}"
                )

                if len(line_inventario_emsys_fiscal) <= 4:
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=f"Erro: Nenhum registro encontrado com a data informada {periodo_btn_buscar}.",
                        status=RpaHistoricoStatusEnum.Falha,
                        tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)],
                    )

                if "no final do" in line_inventario_emsys_fiscal.lower():
                    selecionar_registro_full_path = (
                        "assets\\emsys\\button_selecionar_registro_fiscal.png"
                    )
                    try:
                        button_location = pyautogui.locateCenterOnScreen(
                            selecionar_registro_full_path, confidence=0.6
                        )
                        if button_location:
                            pyautogui.click(button_location)
                            console.print(
                                "Botão 'Selecionar Registro' clicado com sucesso!"
                            )
                            break
                    except Exception as e:
                        return RpaRetornoProcessoDTO(
                            sucesso=False,
                            retorno=f"Erro: Não foi clicar na opção Selecionar Registro no Inventario, erro: {e}.",
                            status=RpaHistoricoStatusEnum.Falha,
                            tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
                        )

                if bool(line_inventario_emsys_fiscal):
                    if (
                        last_line_inventario_emsys_fiscal
                        == line_inventario_emsys_fiscal
                    ):
                        return RpaRetornoProcessoDTO(
                            sucesso=False,
                            retorno=f"Erro: Não foi encontrado a opção do período de busca {periodo_filtro_inicio} nas opções do inventário.",
                            status=RpaHistoricoStatusEnum.Falha,
                            tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)],
                        )
                    else:
                        last_line_inventario_emsys_fiscal = line_inventario_emsys_fiscal
                        pyautogui.press("down")

        else:
            console.print(
                "Mês não é igual a Fevereiro e SPED Inventario é falso, seguindo com a geração do SPED...\n"
            )

        await worker_sleep(3)
        pesquisar_full_path = "assets\\emsys\\button_gerar_sped_fiscal.png"
        try:
            button_location = pyautogui.locateCenterOnScreen(
                pesquisar_full_path, confidence=0.6
            )
            if button_location:
                pyautogui.click(button_location)
                console.print("Botão 'Gerar Sped' clicado com sucesso!")
        except pyautogui.ImageNotFoundException:
            window_rect = main_window.rectangle()
            console.print(f"Area que sera utulizada para o recorte {window_rect}...\n")
            try:
                button_location = pyautogui.locateCenterOnScreen(
                    pesquisar_full_path,
                    region=(
                        window_rect.left,
                        window_rect.top,
                        window_rect.width(),
                        window_rect.height(),
                    ),
                )
                if button_location:
                    button_location = (
                        button_location.x + window_rect.left,
                        button_location.y + window_rect.top,
                    )
                    console.print(
                        f"Botão encontrado nas coordenadas: {button_location}"
                    )
                    pyautogui.click(button_location)
            except pyautogui.ImageNotFoundException:
                console.print(
                    "Erro - Botão de gerar o sped na tela de Gerar Sped Fiscal"
                )
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno="Erro - Botão de gerar o sped na tela de Gerar Sped Fiscal",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
                )
        except Exception as e:
            console.print(
                f"Não foi possivel seguir pois não foi fois possivel interagir com o botão de gerar sped na tela Gerar Sped Fiscal,Error: {e}...\n"
            )
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Não foi possivel seguir pois não foi fois possivel interagir com o botão de gerar sped na tela Gerar Sped Fiscal,Error: {e}",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
            )

        await worker_sleep(12)
        try:
            app = Application().connect(class_name="TMsgBox")
            if "Registro de Entrada" in str(app.windows()):
                console.print(
                    f"Verificando sem possui o pop-up de Registro de Entrada... \n"
                )
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Erro: Livro Fiscal de Registro de Entrada não foi gerado ou não está confirmado para a empresa.",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)],
                )
            elif "Registro de Inventario" in str(app.windows()):
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Erro: POP-Up Registro de Inventario não foi gerado ou não esta confirmado para a empresa",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
                )

            elif "Apuração de ICMS" in str(app.windows()):
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Erro: POP-Up Apuração de ICMS, Livro de Apuração de ICMS não foi gerado no período na empresa",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
                )
            else:
                console.print(f"POP UP: {str(app.windows())}. Continuando...\n")
        except Exception as e:
            console.print(f"Erro ao verificar janelas: {e}. Continuando...\n")

        await worker_sleep(5)
        warning_pop_up = await is_window_open("Aviso")
        if warning_pop_up["IsOpened"] == True:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Erro: POP-Up aviso impedindo seguir para gerar o Sped Fiscal",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
            )

        await worker_sleep(5)
        max_attempts = 100
        i = 0

        while i < max_attempts:
            try:
                app = Application().connect(class_name="TMsgBox")
                if "Informação" in str(app.windows()):
                    main_window = app["Informação"]
                    console.print(f"Tela de Informação encontrada, saindo...\n")
                    break
                elif "Aviso" in str(app.windows()):
                    main_window = app["Aviso"]
                    console.print(f"Tela de Aviso encontrada, saindo...\n")
                    break
            except Exception as e:
                console.print(f"Erro ao verificar janelas: {e}. Continuando...\n")

            warning_pop_up = await is_window_open("Aviso")
            if warning_pop_up["IsOpened"]:
                console.print(
                    f"Tela de Aviso encontrada (via função is_window_open), saindo...\n"
                )
                break
            else:
                console.print(f"Tela de Aviso não encontrada, continuando...\n")

            i += 1
            await worker_sleep(130)

        if i == max_attempts:
            console.print("Número máximo de tentativas atingido. Encerrando...")
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno="Tempo esgotado e numero de tentativas atingido, não foi possivel obter o retorno de conclusão do SPED",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
            )

        # Verificando se possui pop-up de Aviso
        await worker_sleep(6)
        warning_pop_up = await is_window_open("Aviso")
        if warning_pop_up["IsOpened"] == True:
            app = Application().connect(title="Aviso")
            main_window = app["Aviso"]
            main_window.set_focus()

            console.print(f"Obtendo texto do Aviso...\n")
            console.print(
                f"Tirando print da janela do warning para realização do OCR...\n"
            )

            window_rect = main_window.rectangle()
            screenshot = pyautogui.screenshot(
                region=(
                    window_rect.left,
                    window_rect.top,
                    window_rect.width(),
                    window_rect.height(),
                )
            )
            username = getpass.getuser()
            path_to_png = f"C:\\Users\\{username}\\Downloads\\aviso_popup_{short_uuid}_{filial_cod}.png"
            screenshot.save(path_to_png)
            console.print(f"Print salvo em {path_to_png}...\n")

            console.print(
                f"Preparando a imagem para maior resolução e assertividade no OCR...\n"
            )
            image = Image.open(path_to_png)
            image = image.convert("L")
            enhancer = ImageEnhance.Contrast(image)
            image = enhancer.enhance(2.0)
            image.save(path_to_png)
            console.print(f"Imagem preparada com sucesso...\n")
            console.print(f"Realizando OCR...\n")
            captured_text = pytesseract.image_to_string(Image.open(path_to_png))
            console.print(f"Texto Full capturado {captured_text}...\n")
            os.remove(path_to_png)
            if "movimento não permitido" in captured_text.lower():
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Filial: {filial_cod} está com o livro fechado ou encerrado, verificar com o setor fiscal",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)],
                )
            elif "não foi possiv" in captured_text.lower():
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Filial: {filial_cod} Não foi possível gerar o SPED, devido a mensagem: {captured_text}",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)],
                )
            elif "informe o tipo de" in captured_text.lower():
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Mensagem do Aviso, Informe o tipo cobraça ",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)],
                )
            else:
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Aviso não mapeado para seguimento do robo, mensagem: {captured_text}",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
                )
        else:
            console.print(f"POP-up aviso não encontrado...\n")

        # Verifica se a info 'Arquivo do SPED gerado com sucesso' está na tela
        await worker_sleep(6)
        try:
            max_attempts = 10
            i = 0

            while i < max_attempts:
                information_pop_up_class = await is_window_open_by_class(
                    "TMsgBox", "TMsgBox"
                )
                if information_pop_up_class["IsOpened"] == True:
                    break
                else:
                    console.print(f"Aguardando confirmação de sped finalizado...\n")
                    await worker_sleep(5)
                    i += 1

            if i >= max_attempts:
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno="Não foi possivel obter o retorno de conclusão do SPED",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
                )

            console.print("\nArquivo do SPED gerado com sucesso...", style="bold green")
            console.log("Realizando o envio para o Backoffice...\n")
            with open(temp_path_to_txt, "rb") as file:
                file_bytes = io.BytesIO(file.read())

            await worker_sleep(6)
            timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
            desArquivo = f"ArquivoSped_{filial_cod}_{timestamp}.txt"
            try:
                await send_file(
                    historico_id, desArquivo, "txt", file_bytes, file_extension="txt"
                )
                result = (
                    "Arquivo do SPED gerado com sucesso e arquivo salvo no Backoffice !"
                )
                os.remove(temp_path_to_txt)
                return RpaRetornoProcessoDTO(
                    sucesso=True,
                    retorno=result,
                    status=RpaHistoricoStatusEnum.Sucesso,
                )
            except Exception as e:
                result = f"Arquivo do SPED gerado com sucesso, porém gerou erro ao realizar o envio para o backoffice {e} - Arquivo ainda salvo na dispositivo utilizado no diretório {temp_path_to_txt} !"
                console.print(result, style="bold red")
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=result,
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
                )

        except Exception as e:
            console.print(f"Erro ao conectar à janela Informação: {e}\n")
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Erro em obter o retorno, Arquivo do SPED gerado com sucesso, erro {e}",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
            )

    except Exception as ex:
        observacao = f"Erro ao gerar o Arquivo do SPED: {str(ex)}"
        logger.error(observacao)
        console.print(observacao, style="bold red")
        return RpaRetornoProcessoDTO(
            sucesso=False,
            retorno=observacao,
            status=RpaHistoricoStatusEnum.Falha,
            tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)],
        )



import numpy
from worker_automate_hub.models.dto.rpa_historico_request_dto import (
    RpaHistoricoStatusEnum,
    RpaRetornoProcessoDTO,
    RpaTagDTO,
    RpaTagEnum,
)
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import (
    RpaProcessoEntradaDTO,
)
from worker_automate_hub.utils.util import (
    kill_all_emsys,
    worker_sleep,
)
from rich.console import Console
import pyautogui
from datetime import datetime
from dateutil.relativedelta import relativedelta
from PIL import ImageFilter, ImageEnhance
from pytesseract import image_to_string
from pywinauto import Application
import subprocess
import os
from worker_automate_hub.api.client import (
    get_config_by_name,
)
from worker_automate_hub.utils.utils_nfe_entrada import EMSys
import psutil
from time import sleep

pyautogui.PAUSE = 0.5
console = Console()
emsys = EMSys()


def get_text_from_window(window, relative_coords, value=None):
    try:
        screenshot = window.capture_as_image()
        imagem = screenshot.convert("L")
        imagem = imagem.filter(ImageFilter.SHARPEN)
        imagem = ImageEnhance.Contrast(imagem).enhance(2)
        cropped_screenshot = imagem.crop(relative_coords)
        texto = image_to_string(cropped_screenshot, lang="por")
        return (value.upper() in texto.upper()) if value != None else texto.lower()
    except Exception as error:
        console.print(f"Error: {error}")


async def open_contabil_processes():
    try:
        console.print("Abrindo EMSys Contabil...")
        os.startfile("C:\\Users\\automatehub\\Desktop\\Contabil\\contabil1.lnk")
        await worker_sleep(3)
        os.startfile("C:\\Users\\automatehub\\Desktop\\Contabil\\contabil2.lnk")
        await worker_sleep(30)
        os.startfile("C:\\Users\\automatehub\\Desktop\\Contabil\\contabil3.lnk")
        await worker_sleep(20)
        pyautogui.hotkey("win", "d")
        await worker_sleep(4)
        os.startfile("C:\\Users\\automatehub\\Desktop\\Contabil\\contabil4.lnk")
        await worker_sleep(2)
    except Exception as error:
        console.print(f"Error: {error}")


async def integracao_contabil(task: RpaProcessoEntradaDTO) -> RpaRetornoProcessoDTO:
    try:
        await kill_all_emsys()
        await open_contabil_processes()
        config = await get_config_by_name("login_emsys_contabil")

        app = None
        max_attempts = 30
        console.print("Tentando encontrar janela de login...")
        for attempt in range(max_attempts):
            try:
                app = Application(backend="win32").connect(
                    title="Selecione o Usuário para autenticação"
                )
                console.print("Janela encontrada!")
                break
            except:
                console.print("Janela ainda nao encontrada...")
                await worker_sleep(1)
        if not app:
            console.print("Nao foi possivel encontrar a janela de login...")
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno="Erro durante tentativa localizacao de janelas...",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
            )
        emsys.verify_warning_and_error("Erro", "&Ok")
        await worker_sleep(4)
        pyautogui.click(x=1021, y=127)
        console.print("Logando...")
        emsys.verify_warning_and_error("Erro", "&Ok")
        pyautogui.write(config.conConfiguracao.get("user"))
        pyautogui.press("enter")

        await worker_sleep(4)
        pyautogui.write(config.conConfiguracao.get("pass"))
        pyautogui.press("enter")
        await worker_sleep(10)

        main_window = None
        for attempt in range(max_attempts):
            main_window = Application().connect(title="EMSys [Contabil]")
            main_window = main_window.top_window()
            if main_window.exists():
                console.print("Janela encontrada!")
                break
            console.print("Janela ainda nao encontrada...")
            await worker_sleep(1)

        if not main_window:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno="Erro durante tentativa localizacao de janelas....",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
            )

        # Adicionando foco
        try:
            main_window.set_focus()
            console.print(f"Ativando janela: {main_window}")
        except Exception as error:
            console.print(f"Erro ao setar foco na janela: {main_window}")

        await worker_sleep(4)
        console.print("Iniciando interacao a janela de integracao contabil...")
        pyautogui.click(x=566, y=53)
        await worker_sleep(4)
        pyautogui.press("tab", presses=2, interval=1)
        await worker_sleep(8)
        pyautogui.press("enter")
        await worker_sleep(4)
        pyautogui.press("tab")
        pyautogui.click(x=654, y=342)
        await worker_sleep(5)
        console.print("Preenchendo campo periodo...")
        pyautogui.press("tab", presses=2)
        await worker_sleep(4)
        periodo = task.configEntrada.get("periodo")
        periodo = periodo.replace("/", "")
        pyautogui.write(periodo)
        await worker_sleep(5)
        pyautogui.press("tab")
        await worker_sleep(4)
        pyautogui.write(periodo)
        await worker_sleep(3)
        console.print("Selecionando item do campo origem...")
        checkbox_origem = main_window.child_window(class_name="TRzComboBox", found_index=1)
        checkbox_origem.select(task.configEntrada.get("origem"))
        await worker_sleep(3)
        console.print("Clicando no botao pesquisar...")
        pyautogui.click(x=1250, y=380)
        await worker_sleep(20)
        console.print("Aguardando carregamento...")
        max_attempts = 240
        found = False
        console.print("Iniciando verificacoes...")
        for attempt in range(1, max_attempts + 1):
            console.print(
                f"Verificacao {attempt} ( maximo {max_attempts} ) - aguardando 1 minuto..."
            )
            field = get_text_from_window(main_window, (1033, 770, 1301, 785))
            console.print(f"Texto de status tela: {field}")
            if "finalizado" in field:
                found = True
                break
            if "[Pesquisa] - Não é possível realizar a operação em mídias" in field:
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Erro: '[Pesquisa] - Não é possível realizar a operação em mídias', por favor, reenfileirar processo",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
                )
            if "validando" in field and attempt == 1:
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"A integração não foi realizada: nenhum lote para integrar.",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
                )
            await worker_sleep(30)
        if not found:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno="Erro durante a pesquisa de lancamentos ( max_attempts excedido )",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
            )
        await worker_sleep(4)
        dados_consistentes = False
        dados_consistentes_is_checked = False
        max_attempts_dados_consistentes = 20
        console.print("Verificando ocorrencia de inconsistencias...")
        for attempt in range(1, max_attempts_dados_consistentes + 1):
            console.print(f"Verificacao {attempt}...")
            try:
                total_debito = main_window.Edit3.window_text()
                total_credito = main_window.Edit2.window_text()
                diferenca = main_window.Edit.window_text()
                if total_debito != total_credito:
                    if not dados_consistentes_is_checked:
                        pyautogui.click(x=702, y=758)  # lotes consistentes
                        await worker_sleep(3)
                        dados_consistentes_is_checked = True
                        total_debito = main_window.Edit3.window_text()
                        total_credito = main_window.Edit2.window_text()
                        diferenca = main_window.Edit.window_text()
                        if total_debito == total_credito:
                            if total_debito == "0,00":
                                await worker_sleep(2)
                                console.print("Clicando em integrar lançamentos...")
                                pyautogui.click(x=947, y=790)  # integrar lancamentos
                                await worker_sleep(2)
                                return RpaRetornoProcessoDTO(
                                    sucesso=False,
                                    retorno="Integração não realizada: nenhum lote para integrar após selecionar lotes consistentes",
                                    status=RpaHistoricoStatusEnum.Falha,
                                    tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)],
                                )
                            else:
                                await worker_sleep(2)
                                console.print("Clicando em integrar lançamentos...")
                                pyautogui.click(x=947, y=790)  # integrar lancamentos
                        else:

                            try:
                                diferenca_formatada = (
                                    diferenca.replace(".", "").replace(",", ".").strip()
                                )

                                diferenca_numerica = float(diferenca_formatada)

                                if diferenca_numerica <= 0.10:
                                    await worker_sleep(2)
                                    console.print("Clicando em integrar lançamentos...")
                                    pyautogui.click(x=947, y=790)
                                else:
                                    return RpaRetornoProcessoDTO(
                                        sucesso=False,
                                        retorno="Integração não realizada, pois os valores de crédito e débito divergem mesmo após clicar em lotes consistentes. Dif. superior a 0,10.",
                                        status=RpaHistoricoStatusEnum.Falha,
                                        tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)],
                                    )

                            except ValueError:
                                return RpaRetornoProcessoDTO(
                                    sucesso=False,
                                    retorno="Integração não realizada, pois os valores de crédito e débito divergem  após clicar em lotes consistentes. Não foi possível validar se Dif. é superior a 0,10",
                                    status=RpaHistoricoStatusEnum.Falha,
                                    tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)],
                                )

                    else:
                        return RpaRetornoProcessoDTO(
                            sucesso=False,
                            status=RpaHistoricoStatusEnum.Falha,
                            tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)],
                        )

                    await worker_sleep(50)

                else:
                    await worker_sleep(2)
                    console.print("Clicando em integrar lançamentos...")
                    pyautogui.click(x=947, y=790)  # integrar lancamentos

                if total_credito == total_debito and diferenca == "0,00":
                    dados_consistentes = True
                    break
            except:
                console.print(
                    "Nao foi possivel encontrar os campos diretamente, tentando via ocr..."
                )
                total_debito = get_text_from_window(
                    main_window, (854, 722, 956, 747)
                ).replace("\n", "")
                await worker_sleep(1)
                total_credito = get_text_from_window(
                    main_window, (1055, 722, 1162, 747)
                ).replace("\n", "")
                await worker_sleep(1)
                diferenca = get_text_from_window(
                    main_window, (1233, 722, 1336, 747)
                ).replace("\n", "")
                await worker_sleep(1)
                if total_debito != total_credito or diferenca != "0,00":
                    if not dados_consistentes_is_checked:
                        pyautogui.click(x=702, y=758)  # lotes consistentes
                        dados_consistentes_is_checked = True
                    await worker_sleep(2)
                    console.print("Clicando em integrar lançamentos...")
                    pyautogui.click(x=947, y=790)  # integrar lancamentos
                    await worker_sleep(60)
        if not dados_consistentes:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno="Erro: lotes inconsistentes (maximo de tentativas excedido em dados consistentes)",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)],
            )
        tela_processo_finalizado = None
        main_window_capture = main_window.capture_as_image()
        console.print("Aguardando a finalizacao do processo...")
        for attempt in range(1, max_attempts + 1):
            console.print("Aguardando 1 minuto...")
            await worker_sleep(60)
            new_window_capture = main_window.capture_as_image()
            array_image1 = numpy.array(main_window_capture)
            array_image2 = numpy.array(new_window_capture)
            if not numpy.array_equal(array_image1, array_image2):
                tela_processo_finalizado = new_window_capture
                break

        mensagem_mapeada = get_text_from_window(main_window, (830, 513, 998, 546))
        console.print(f"Mensagem de erro extraída da tela: {mensagem_mapeada}")
        if "conta indefinida" in mensagem_mapeada:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Integração não realizada, existe um erro de conta indefinida",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)],
            )

        if tela_processo_finalizado and dados_consistentes_is_checked:
            await worker_sleep(3)
            pyautogui.click(x=951, y=573)  # clicando em ok
            await worker_sleep(3)
            pyautogui.click(x=702, y=758)
            await worker_sleep(10)

            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Integração realizada, porém, existem LOTES INCONSISTENTES",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)],
            )
        elif tela_processo_finalizado and not dados_consistentes_is_checked:
            return RpaRetornoProcessoDTO(
                sucesso=True,
                retorno=f"Sucesso ao executar processo de integracao contabil",
                status=RpaHistoricoStatusEnum.Sucesso,
            )

        if not tela_processo_finalizado:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Erro de 'timeout' durante o processo integração contabil...",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
            )

    except Exception as erro:
        return RpaRetornoProcessoDTO(
            sucesso=False,
            retorno=f"Erro durante o processo integração contabil, erro : {erro}",
            status=RpaHistoricoStatusEnum.Falha,
            tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
        )



import asyncio
import getpass
import warnings
import os
import re
import uuid
import time
import win32clipboard
import difflib

import pyautogui
import pytesseract
from datetime import datetime, timedelta
from pywinauto.application import Application
from PIL import Image, ImageEnhance, ImageFilter
from pywinauto.keyboard import send_keys
import win32clipboard
from pywinauto_recorder.player import set_combobox
from rich.console import Console
from worker_automate_hub.api.ahead_service import save_xml_to_downloads
from worker_automate_hub.api.client import (
    get_config_by_name,
    get_status_nf_emsys,
)
from worker_automate_hub.models.dto.rpa_historico_request_dto import (
    RpaHistoricoStatusEnum,
    RpaRetornoProcessoDTO,
    RpaTagDTO,
    RpaTagEnum,
)
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import (
    RpaProcessoEntradaDTO,
)
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.util import (
    delete_xml,
    error_after_xml_imported,
    get_xml,
    carregamento_import_xml,
    import_nfe,
    is_window_open,
    is_window_open_by_class,
    itens_not_found_supplier,
    kill_all_emsys,
    login_emsys,
    select_documento_type,
    set_variable,
    type_text_into_field,
    warnings_after_xml_imported,
    worker_sleep,
    check_nota_importada,
)
from worker_automate_hub.utils.utils_nfe_entrada import EMSys

pyautogui.PAUSE = 0.5
pyautogui.FAILSAFE = False
console = Console()

emsys = EMSys()


async def entrada_de_notas_9(task: RpaProcessoEntradaDTO) -> RpaRetornoProcessoDTO:
    """
    Processo que relazia entrada de notas no ERP EMSys(Linx).

    """
    try:
        # Get config from BOF
        config = await get_config_by_name("login_emsys")
        console.print(task)

        # Seta config entrada na var nota para melhor entendimento
        nota = task.configEntrada
        multiplicador_timeout = int(float(task.sistemas[0].timeout))
        set_variable("timeout_multiplicador", multiplicador_timeout)

        # Fecha a instancia do emsys - caso esteja aberta
        await kill_all_emsys()

        # Download XML
        console.log("Realizando o download do XML..\n")
        await save_xml_to_downloads(nota["nfe"])

        app = Application(backend="win32").start("C:\\Rezende\\EMSys3\\EMSys3.exe")
        warnings.filterwarnings(
            "ignore",
            category=UserWarning,
            message="32-bit application should be automated using 32-bit Python",
        )
        console.print("\nEMSys iniciando...", style="bold green")
        return_login = await login_emsys(config.conConfiguracao, app, task)

        if return_login.sucesso == True:
            type_text_into_field(
                "Nota Fiscal de Entrada", app["TFrmMenuPrincipal"]["Edit"], True, "50"
            )
            pyautogui.press("enter")
            await worker_sleep(2)
            pyautogui.press("enter")
            console.print(
                f"\nPesquisa: 'Nota Fiscal de Entrada' realizada com sucesso",
                style="bold green",
            )
        else:
            logger.info(f"\nError Message: {return_login.retorno}")
            console.print(f"\nError Message: {return_login.retorno}", style="bold red")
            return return_login

        await worker_sleep(6)

        # Procura campo documento
        console.print("Navegando pela Janela de Nota Fiscal de Entrada...\n")
        document_type = await select_documento_type(
            "NOTA FISCAL DE ENTRADA ELETRONICA - DANFE"
        )
        if document_type.sucesso == True:
            console.log(document_type.retorno, style="bold green")
        else:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=document_type.retorno,
                status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

        await worker_sleep(4)

        # Clica em 'Importar-Nfe'
        imported_nfe = await import_nfe()
        if imported_nfe.sucesso == True:
            console.log(imported_nfe.retorno, style="bold green")
        else:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=imported_nfe.retorno,
                status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

        await worker_sleep(5)

        await get_xml(nota.get("nfe"))
        await worker_sleep(3)

        # VERIFICANDO A EXISTENCIA DE WARNINGS
        warning_pop_up = await is_window_open("Warning")
        if warning_pop_up["IsOpened"] == True:
            warning_work = await warnings_after_xml_imported()
            if warning_work.sucesso == True:
                console.log(warning_work.retorno, style="bold green")
            else:
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=warning_work.retorno,
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                )

        # VERIFICANDO A EXISTENCIA DE ERRO
        erro_pop_up = await is_window_open("Erro")
        if erro_pop_up["IsOpened"] == True:
            error_work_message = await error_after_xml_imported()
            return RpaRetornoProcessoDTO(
                sucesso=error_work_message.sucesso,
                retorno=error_work_message.retorno,
                status=error_work_message.status,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

        app = Application().connect(
            title="Informações para importação da Nota Fiscal Eletrônica"
        )
        main_window = app["Informações para importação da Nota Fiscal Eletrônica"]


        # INTERAGINDO COM A DATA DE ENTRADA
        await worker_sleep(2)
        try:
            recebimento_fisico = nota.get("recebimentoFisico", None)
            if recebimento_fisico:
                recebimento_fisico = nota["recebimentoFisico"].split(" ")
                pyautogui.write(recebimento_fisico[0])
                await worker_sleep(2)
        except:
            console.print(
                f"A chave recebimentoFisico não está presente na config de entrada...\n"
            )

        # INTERAGINDO COM A NATUREZA DA OPERACAO
        cfop = int(nota.get("cfop"))
        console.print(f"Inserindo a informação da CFOP, caso se aplique {cfop} ...\n")
        if cfop == 5655:
            combo_box_natureza_operacao = main_window.child_window(
                class_name="TDBIComboBox", found_index=0
            )
            combo_box_natureza_operacao.click()

            await worker_sleep(3)
            try:
                set_combobox("||List", "1652-COMPRA DE MERCADORIAS- 1.652")
            except:
                console.log("Não foi possivel selecionar o tipo de documento via set combobox, realizando a alteração utilizando pyautogui write")
                pyautogui.write("1102")
                await worker_sleep(2)
                pyautogui.hotkey("enter")
                await worker_sleep(2)
                pyautogui.write("1652-COMPRA DE MERCADORIAS- 1.652")
                await worker_sleep(2)

        elif cfop == 6655:
            combo_box_natureza_operacao = main_window.child_window(
                class_name="TDBIComboBox", found_index=0
            )
            combo_box_natureza_operacao.click()

            await worker_sleep(3)
            try:
                set_combobox("||List", "2652-COMPRA DE MERCADORIAS- 2.652")
            except:
                console.log("Não foi possivel selecionar o tipo de documento via set combobox, realizando a alteração utilizando pyautogui write")
                pyautogui.write("1102")
                await worker_sleep(2)
                pyautogui.hotkey("enter")
                await worker_sleep(2)
                pyautogui.write("2652-COMPRA DE MERCADORIAS- 2.652")
                await worker_sleep(2)
        else:
            console.print(
                "Erro mapeado, CFOP diferente de 5655 ou 6655, necessario ação manual ou ajuste no robo...\n"
            )
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno="Erro mapeado, CFOP diferente de 5655 ou 6655, necessario ação manual ou ajuste no robo",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)]
            )
        

        combo_box_natureza_operacao = main_window.child_window(class_name="TDBIComboBox", found_index=0)
        if "2652-COMPRA DE MERCADORIAS- 2.652" in combo_box_natureza_operacao.window_text() or "1652-COMPRA DE MERCADORIAS- 1.652" in combo_box_natureza_operacao.window_text():
            console.log("CFOP informado com sucesso")
        else:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno="Não foi possivel selecionar o CFOP",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

        await worker_sleep(3)
        try:
            console.print(f"Trabalhando com itens com multiplas referencias.\n")
            itens_nao_semelhantes = []
            # TRABALHANDO COM ITENS QUE POSSUEM MULTIPLAS REFERÊNCIAS
            while True:
                await worker_sleep(3)
                console.print(
                    f"Verificando a existencia de fornecedor com múltiplas referencias...\n"
                )
                itens_by_supplier = await is_window_open_by_class(
                    "TFrmSelecionaItensFornecedor", "TFrmSelecionaItensFornecedor"
                )
                if itens_by_supplier["IsOpened"] == True:
                    new_app = Application(backend="uia").connect(
                        class_name="TFrmSelecionaItensFornecedor"
                    )
                    window = new_app["TFrmSelecionaItensFornecedor"]
                    window.set_focus()
                    await worker_sleep(2)

                    console.print(f"Obtendo item com multiplas referências...\n")
                    console.print(f"Tirando print da janela para realização do OCR...\n")

                    text_captured = False
                    count_while = 0
                    max_attempts = 3
                    item_da_nota = ''

                    while count_while < max_attempts:
                        window_rect = window.rectangle()
                        console.print(f"Area que sera utulizada para o screenshot {window_rect}...\n")
                        screenshot = window.capture_as_image()

                        username = getpass.getuser()
                        short_uuid = str(uuid.uuid4()).replace('-', '')[:6]
                        path_to_png = f"C:\\Users\\{username}\\Downloads\\{short_uuid}.png"
                        screenshot.save(path_to_png)
                        while not os.path.exists(path_to_png) or os.path.getsize(path_to_png) == 0:
                            time.sleep(0.1)
                        console.print(f"Print salvo em {path_to_png}...\n")

                        await worker_sleep(2)
                        console.print("Preparando a imagem para maior resolução e assertividade no OCR...\n")
                        image = Image.open(path_to_png)
                        image = image.convert("L")
                        enhancer = ImageEnhance.Contrast(image)
                        image = enhancer.enhance(2.0)
                        image.save(path_to_png)
                        console.print("Imagem preparada com sucesso...\n")

                        console.print(f"Imagem antes do OCR: {image}\n")
                        console.print(f"Dimensões da imagem: {image.size}\n")

                        console.print("Realizando OCR...\n")
                        captured_text = pytesseract.image_to_string(image)
                        console.print(f"Texto Full capturado {captured_text}, tentando obter o item da nota...\n")

                        match = re.search(r"Item da Nota:\s*(.*)\s*", captured_text)
                        if os.path.exists(path_to_png):
                            os.remove(path_to_png)
                            console.print(f"Imagem apagada com sucesso do diretorio {path_to_png}... \n")
                        else:
                            console.print(f"Imagem não encontrada para realização do OCR... \n")

                        console.print(f"Texto extraido do RegEx: {match}... \n")
                        if match:
                            item_da_nota = match.group(1).strip()
                            console.print(f"Item da Nota capturado: {item_da_nota}... \n")
                            text_captured = True
                            break
                        else:
                            if match:
                                item_da_nota = match.group(1).strip()
                                console.print(f"Item da Nota capturado: {item_da_nota}... \n")
                                text_captured = True
                                break
                            else:
                                match = re.search(r"Item da (Nota|Nata|N0ta)\s*(.*)\s*", captured_text)
                                if match:
                                    item_da_nota = match.group(1).strip()
                                    console.print(f"Item da Nota capturado: {item_da_nota}... \n")
                                    text_captured = True
                                    break
                                else:
                                    console.print(f"Tentativa {count_while + 1} de {max_attempts} falhou. Tentando novamente...\n")
                                    count_while += 1

                    if not text_captured:
                        return RpaRetornoProcessoDTO(
                            sucesso=False,
                            retorno="Quantidade de tentativa atingida (3), não foi possivel capturar o item da nota com multiplas referencias para andamento no processo",
                            status=RpaHistoricoStatusEnum.Falha,
                            tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                        )

                    console.print(
                        f"Interagindo com os multiplos itens e detectando qual ira ser selecionado para andamento no processo... \n"
                    )
                    copied_codes_list = []
                    count = 0
                    last_copied = None

                    while True:
                        if count == 0:
                            send_keys("^({HOME})")
                        else:
                            send_keys("{DOWN 1}")

                        send_keys("^c")
                        win32clipboard.OpenClipboard()
                        window_message = win32clipboard.GetClipboardData().strip()
                        win32clipboard.CloseClipboard()

                        console.print(f"Linha copiada: {window_message}... \n")

                        if last_copied is not None and window_message == last_copied:
                            break

                        copied_codes_list.append(
                            {"linha": count, "conteudo": window_message}
                        )

                        last_copied = window_message
                        count = +1

                    console.print(f"Valores copiados {copied_codes_list}... \n")

                    extracted_items = {}

                    for entry in copied_codes_list:
                        content = entry["conteudo"]
                        lines = content.split("\r\n")
                        for line in lines:
                            items = line.split("\t")
                            if len(items) >= 3:
                                cod_item = items[0].strip()
                                descricao = items[1].strip()
                                cod_barra = items[2].strip()

                                extracted_items[cod_item] = {
                                    "Descrição": descricao,
                                    "Código de Barra": cod_barra,
                                    "Linha": entry["linha"],
                                }

                    best_match = None
                    highest_ratio = 0.0

                    for cod_item, info in extracted_items.items():
                        descricao = info["Descrição"]
                        similarity_ratio = difflib.SequenceMatcher(
                            None, item_da_nota, descricao
                        ).ratio()

                        if similarity_ratio > highest_ratio:
                            highest_ratio = similarity_ratio
                            best_match = {
                                "Cod Item": cod_item,
                                "Descrição": descricao,
                                "Código de Barra": info["Código de Barra"],
                                "Linha": info["Linha"],
                                "Similaridade": similarity_ratio,
                            }

                    if best_match and highest_ratio > 0.7:
                        console.print(
                            f"Melhor semelhança encontrada {best_match}, selecionando... \n"
                        )
                        send_keys("^({HOME})")
                        send_keys("{DOWN " + str(best_match["Linha"]) + "}")
                        send_keys("%o")
                        await worker_sleep(2)
                    else:
                        itens_nao_semelhantes.append(
                            {"item": item_da_nota, "match": best_match}
                        )
                        send_keys("%o")
                else:
                    console.print(
                        "Não possui a tela de fornecedor com múltiplas referencias, seguindo com o processo...\n"
                    )
                    break

            if len(itens_nao_semelhantes) > 0:
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Não foi possivel encontrar o item mais proximo ao item da nota com multiplas referencias {itens_nao_semelhantes}",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                )
        except Exception as error:
            console.print("Erro durante a trativa de multiplas referencias, erro : {error}")
        
        await worker_sleep(3)
        # INTERAGINDO COM O CAMPO ALMOXARIFADO
        filialEmpresaOrigem = nota.get("filialEmpresaOrigem")
        console.print(
            f"Inserindo a informação do Almoxarifado {filialEmpresaOrigem} ...\n"
        )
        try:
            new_app = Application(backend="uia").connect(
                title="Informações para importação da Nota Fiscal Eletrônica"
            )
            window = new_app["Informações para importação da Nota Fiscal Eletrônica"]
            edit = window.child_window(
                class_name="TDBIEditCode", found_index=3, control_type="Edit"
            )
            valor_almoxarifado = filialEmpresaOrigem + "50"
            edit.set_edit_text(valor_almoxarifado)
            edit.type_keys("{TAB}")
        except Exception as e:
            console.print(f"Erro ao iterar itens de almoxarifado: {e}")
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Erro ao iterar itens de almoxarifado: {e}",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

        await worker_sleep(1)
        console.print("Clicando em OK... \n")

        max_attempts = 6
        i = 0
        while i < max_attempts:
            console.print("Clicando no botão de OK...\n")
            try:
                try:
                    btn_ok = main_window.child_window(title="Ok")
                    btn_ok.click()
                except:
                    btn_ok = main_window.child_window(title="&Ok")
                    btn_ok.click()
            except:
                console.print("Não foi possivel clicar no Botão OK... \n")

            await worker_sleep(3)

            console.print(
                "Verificando a existencia da tela Informações para importação da Nota Fiscal Eletrônica...\n"
            )

            try:
                informacao_nf_eletronica = await is_window_open(
                    "Informações para importação da Nota Fiscal Eletrônica"
                )
                if informacao_nf_eletronica["IsOpened"] == False:
                    console.print(
                        "Tela Informações para importação da Nota Fiscal Eletrônica fechada, seguindo com o processo"
                    )
                    break
            except Exception as e:
                console.print(
                    f"Tela Informações para importação da Nota Fiscal Eletrônica encontrada. Tentativa {i + 1}/{max_attempts}."
                )

            i += 1

        if i == max_attempts:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Número máximo de tentativas atingido, Não foi possivel finalizar os trabalhos na tela de Informações para importação da Nota Fiscal Eletrônica",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

        await worker_sleep(2)
        waiting_for_delay = await carregamento_import_xml()
        if waiting_for_delay.sucesso:
            console.print(waiting_for_delay.retorno)
        else:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=waiting_for_delay.retorno,
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

        try:
            console.print("Verificando itens não localizados ou NCM...\n")
            itens_by_supplier = await is_window_open_by_class("TFrmAguarde", "TMessageForm")

            if itens_by_supplier["IsOpened"] == True:
                itens_by_supplier_work = await itens_not_found_supplier(nota.get("nfe"))

                if not itens_by_supplier_work.sucesso:
                    return itens_by_supplier_work

        except Exception as error:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Falha ao verificar a existência de POP-UP de itens não localizados: {error}",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

        console.print("Navegando pela Janela de Nota Fiscal de Entrada...\n")
        app = Application().connect(class_name="TFrmNotaFiscalEntrada")
        main_window = app["TFrmNotaFiscalEntrada"]

        main_window.set_focus()
        await worker_sleep(1)
        console.print("Acessando os itens da nota... \n")
        panel_TPage = main_window.child_window(class_name="TPage", title="Formulario")
        panel_TTabSheet = panel_TPage.child_window(class_name="TcxCustomInnerTreeView")
        panel_TTabSheet.wait("visible")
        panel_TTabSheet.click()
        send_keys("{DOWN " + ("5") + "}")

        # CONFIRMANDO SE A ABA DE ITENS FOI ACESSADA COM SUCESSO
        try:
            panel_TPage = main_window.child_window(class_name="TPage", title="Formulario")
            panel_TPage.wait("visible")
            panel_TTabSheet = panel_TPage.child_window(class_name="TTabSheet")
            title_n_serie = panel_TPage.child_window(title="N° Série")

            console.print("Verificando se os itens foram abertos com sucesso... \n")
            if not title_n_serie:
                console.print(f"Não foi possivel acessar a aba de 'Itens da nota...\n")
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno="Não foi possivel acessar a aba de 'Itens da nota'",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                )
        except Exception as e:
            return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Não foi possivel acessar a aba de 'Itens da nota', erro: {e}",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                )

        await worker_sleep(2)

        observacoes_nota = nota.get("observacoes")
        pattern = rf"(\b{filialEmpresaOrigem}\d+)\s*-\s*TANQUE\s+(\d+)\s*[/-]\s*(\w+\s*\w*)\s*\((.*?)\)"

        resultados_itens_ahead = re.findall(pattern, observacoes_nota)

        list_distribuicao_obs = []
        for codigo_filial, numero_tanque, tipo_combustivel, valor in resultados_itens_ahead:
            tanque =f"{codigo_filial} - {numero_tanque} - {tipo_combustivel} ({valor})"
            list_distribuicao_obs.append(tanque)


        if len(list_distribuicao_obs) > 0:
            console.print(f'Distribuição observação a serem processados: {list_distribuicao_obs}')
            index_tanque = 0
            list_tanques_distribuidos = []
            send_keys("{TAB 2}", pause=0.1)
            
            try:
                for info_distribuicao_obs in list_distribuicao_obs:
                    await worker_sleep(2)
                    console.print(f"Tanque a ser distribuido: {info_distribuicao_obs}... \n")
                    send_keys("^({HOME})")
                    await worker_sleep(1)
                    send_keys("{DOWN " + str(index_tanque) + "}", pause=0.1)
                    await worker_sleep(1)
                    send_keys("+{F10}")
                    await worker_sleep(1)
                    send_keys("{DOWN 6}")
                    await worker_sleep(1)
                    send_keys("{ENTER}")
                    await worker_sleep(4)

                    
                    max_attempts = 5
                    i = 0
                    while i < max_attempts:
                        distribuir_item_window = await is_window_open("Distribui Item Tanque")
                        if distribuir_item_window["IsOpened"] == True:
                            app = Application().connect(title="Distribui Item Tanque")
                            main_window = app["Distribui Item Tanque"]

                            main_window.set_focus()
                            break
                        else:
                            await worker_sleep(3)
                            i = i + 1
                    
                    if i >= max_attempts:
                        return RpaRetornoProcessoDTO(
                            sucesso=False,
                            retorno=f"Erro ao trabalhar nas alterações dos item de tanque, tela de Distribui item tanque não foi encontrada",
                            status=RpaHistoricoStatusEnum.Falha,
                            tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                        )

                    try:
                        panel_grid = main_window.child_window(class_name="TcxGridSite")
                    except:
                        panel_grid = main_window.child_window(class_name="TcxGrid")

                    grid_rect = panel_grid.rectangle()
                    center_x = (grid_rect.left + grid_rect.right) // 2
                    center_y = (grid_rect.top + grid_rect.bottom) // 2

                    pyautogui.click(center_x, center_y)
                    await worker_sleep(1)
                    send_keys("^({HOME})")
                    await worker_sleep(1)
                    send_keys("{LEFT 3}")

                    distribuiu_algo = False
                    distribuicao_atual = []
                    last_line_almoxarifado_emsys = 'x'
                    max_distribuicao = 0

                    while max_distribuicao <= 20:
                        console.print(f"Tentativa: {max_distribuicao}... \n")
                        await worker_sleep(1)
                        with pyautogui.hold('ctrl'):
                            pyautogui.press('c')

                        await worker_sleep(1)

                        with pyautogui.hold('ctrl'):
                            pyautogui.press('c')

                        win32clipboard.OpenClipboard()
                        line_almoxarifado_emsys = win32clipboard.GetClipboardData().strip()
                        win32clipboard.CloseClipboard()
                        console.print(f"Linha atual copiada do Emsys: {line_almoxarifado_emsys}\nUltima Linha copiada: {last_line_almoxarifado_emsys}")

                        if bool(line_almoxarifado_emsys):
                            if last_line_almoxarifado_emsys == line_almoxarifado_emsys:
                                break
                            else:
                                last_line_almoxarifado_emsys = line_almoxarifado_emsys

                            codigo_almoxarifado_emsys = line_almoxarifado_emsys.split('\n')[1].split('\t')[0].strip()

                            for second_info_distribuicao_obs in list_distribuicao_obs:
                                codigo_almoxarifado_obs = second_info_distribuicao_obs.split('-')[0].strip()
                                console.print(
                                    f"Código almoxarifado emsys: {codigo_almoxarifado_emsys}\nCodigo almoxarifado obs: {codigo_almoxarifado_obs}",
                                    None)
                                if codigo_almoxarifado_obs == codigo_almoxarifado_emsys and not second_info_distribuicao_obs in list_tanques_distribuidos:
                                    console.print("Entrou no IF para distribuir tanques.")
                                    console.print(
                                        f"Linha atual copiada do Emsys: {line_almoxarifado_emsys}\nUltima Linha copiada: {last_line_almoxarifado_emsys}")
                                    quantidade_combustivel = re.findall(r'\((.*?)\)', second_info_distribuicao_obs)[0].replace('.', '')

                                    send_keys("{LEFT 3}")
                                    await worker_sleep(1)
                                    send_keys("{RIGHT 3}")

                                    pyautogui.press('enter')
                                    await worker_sleep(1)
                                    pyautogui.write(quantidade_combustivel)
                                    pyautogui.press('enter')
                                    list_tanques_distribuidos.append(second_info_distribuicao_obs)
                                    distribuicao_atual.append(f"Valor do tipo de combustivel:{codigo_almoxarifado_emsys} nas observações é {quantidade_combustivel}")
                                    distribuiu_algo = True

                        max_distribuicao = max_distribuicao + 1
                        pyautogui.press('down')
                        await worker_sleep(1)

                    index_tanque = index_tanque + 1
                    console.print(f"Index Tanque: {index_tanque}")

                    if distribuiu_algo:
                        console.print(f"Extraindo informação da Janela de Distribuir Itens para realização do OCR...\n")

                        max_attempts = 3
                        attempts = 0
                        captured_text = None
                        resultado = None
                        while attempts < max_attempts:
                            main_window.set_focus()
                            window_rect = main_window.rectangle()
                            screenshot = pyautogui.screenshot(
                                region=(
                                    window_rect.left,
                                    window_rect.top,
                                    window_rect.width(),
                                    window_rect.height(),
                                )
                            )
                            username = getpass.getuser()
                            path_to_png = f"C:\\Users\\{username}\\Downloads\\distribuir_iten_{nota.get("nfe")}.png"
                            screenshot.save(path_to_png)
                            console.print(f"Print salvo em {path_to_png}...\n")

                            console.print(
                                f"Preparando a imagem para maior resolução e assertividade no OCR...\n"
                            )
                            image = Image.open(path_to_png)
                            image = image.filter(ImageFilter.SHARPEN)
                            image = image.convert("L")
                            enhancer = ImageEnhance.Contrast(image)
                            image = enhancer.enhance(2.0)
                            image.save(path_to_png)
                            console.print(f"Imagem preparada com sucesso...\n")
                            console.print(f"Realizando OCR...\n")
                            await worker_sleep(1)
                            captured_text = pytesseract.image_to_string(Image.open(path_to_png))
                            console.print(f"Texto Full capturado {captured_text}...\n")
                            os.remove(path_to_png)

                            pattern_qtd_restante = r"Quantidade restante:\s([\d,]+)"
                            resultado = re.search(pattern_qtd_restante, captured_text)
                            if resultado:
                                break
                            else:
                                await worker_sleep(2)
                                console.print(f"Não conseguiu encontrar a quantidade restante, tentando cortar a imagem...\n")
                                width, height = image.size
                                box = (0, height - 100, width, height)
                                cropped_image = image.crop(box)
                                
                                cropped_image = cropped_image.convert("L")
                                cropped_image = cropped_image.filter(ImageFilter.SHARPEN)
                                enhancer = ImageEnhance.Contrast(cropped_image)
                                cropped_image = enhancer.enhance(2.0)
                                
                                captured_text = pytesseract.image_to_string(cropped_image)
                                console.print(f"Texto Full capturado {captured_text}...\n")

                                resultado = re.search(pattern_qtd_restante, captured_text)
                                if resultado:
                                    break

                                await worker_sleep(2)
                                attempts += 1

                        try:
                            if resultado:
                                quantidade_restante = int(resultado.group(1).replace(",", ""))
                            
                                if quantidade_restante > 0:
                                    return RpaRetornoProcessoDTO(
                                        sucesso=False,
                                        retorno=f"A distribuição informada diverge do que está na nota, {', '.join(distribuicao_atual)} e o valor da quantidade restante é de {str(quantidade_restante)}.",
                                        status=RpaHistoricoStatusEnum.Falha,
                                        tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)]
                                    )
                                else:
                                    console.print(f"A quantidade restante é igual ou menor que 0, seguindo... \n")
                            else:
                                return RpaRetornoProcessoDTO(
                                    sucesso=False,
                                    retorno=f"Não foi possivel obter o resultado da quantidade restante, texto extraido: {captured_text}",
                                    status=RpaHistoricoStatusEnum.Falha,
                                    tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)]
                                )
                        except Exception as e:
                            return RpaRetornoProcessoDTO(
                                sucesso=False,
                                retorno=f"Erro {e}, Não foi possivel obter o resultado da quantidade restante",
                                status=RpaHistoricoStatusEnum.Falha,
                                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                            )

                        console.print(f"Algum Item foi distribuido, clicando em OK para salvar")
                        main_window.set_focus()
                        await worker_sleep(1)
                        btn_ok = main_window.child_window(
                                class_name="TBitBtn", found_index=1
                            )
                        btn_ok.click()
                        await worker_sleep(5)
                        
                        max_attempts = 5
                        i = 0
                        while i < max_attempts:
                            distribuir_item_window = await is_window_open("Distribui Item Tanque")
                            if distribuir_item_window["IsOpened"] == True:
                                try:
                                    btn_ok.click()
                                except:
                                    console.print(f"Tela de distribuir item deve ter sido encerrada")
                                finally:
                                    i = i + 1
                                    await worker_sleep(2)
                            else:
                                console.print(f"Tela de distribuir item do tanque finalizado com sucesso")
                                await worker_sleep(5)
                                break
                        
                        if i >= max_attempts:
                            return RpaRetornoProcessoDTO(
                                sucesso=False,
                                retorno=f"Tela de distribuir item não foi encerrada",
                                status=RpaHistoricoStatusEnum.Falha,
                                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                            )
                    else:
                        console.print(f"Nenhum item foi distribuido, clicando em Cancelar")
                        btn_cancelar = main_window.child_window(
                                class_name="TBitBtn", found_index=0
                            )
                        btn_cancelar.click()

                        max_attempts = 5
                        i = 0
                        while i < max_attempts:
                            distribuir_item_window = await is_window_open("Distribui Item Tanque")
                            if distribuir_item_window["IsOpened"] == True:
                                try:
                                    btn_cancelar.click()
                                except:
                                    console.print(f"Tela de distribuir item deve ter sido encerrada")
                                finally:
                                    i = i + 1
                                    await worker_sleep(2)
                            else:
                                console.print(f"Tela de distribuir item do tanque finalizado com sucesso")
                                await worker_sleep(5)
                                break
                        
                        if i >= max_attempts:
                            return RpaRetornoProcessoDTO(
                                sucesso=False,
                                retorno=f"Tela de distribuir item não foi encerrada",
                                status=RpaHistoricoStatusEnum.Falha,
                                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                            )
            except Exception as e:
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Erro ao trabalhar nas alterações dos itens: {e}",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                )

        else:
            console.print("Nenhum item com necessidade de ser alterado... \n")

        await worker_sleep(5)
        
        try:
            console.print("Verificando itens não localizados ou NCM...\n")
            itens_by_supplier = await is_window_open_by_class("TFrmAguarde", "TMessageForm")

            if itens_by_supplier["IsOpened"] == True:
                itens_by_supplier_work = await itens_not_found_supplier(nota.get("nfe"))

                if not itens_by_supplier_work.sucesso:
                    return itens_by_supplier_work

        except Exception as error:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Falha ao verificar a existência de POP-UP de itens não localizados: {error}",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

        console.print("Navegando pela Janela de Nota Fiscal de Entrada...\n")
        app = Application().connect(class_name="TFrmNotaFiscalEntrada")
        main_window = app["TFrmNotaFiscalEntrada"]

        main_window.set_focus()
        console.log("Seleciona Pagamento", style="bold yellow")
        try:
            pyautogui.click(623, 374)
            await worker_sleep(1)
            send_keys("{DOWN " + ("4") + "}")
        except Exception as e:
            panel_TPage = main_window.child_window(class_name="TPage", title="Formulario")
            panel_TTabSheet = panel_TPage.child_window(class_name="TcxCustomInnerTreeView")
            panel_TTabSheet.wait("visible")
            panel_TTabSheet.click()
            send_keys("{DOWN " + ("2") + "}")

        await worker_sleep(6)
        try:
            panel_TPage = main_window.child_window(class_name="TPage", title="Formulario")
            panel_TTabSheet = panel_TPage.child_window(class_name="TPageControl")

            panel_TabPagamento = panel_TTabSheet.child_window(class_name="TTabSheet")
            panel_TabParcelamento = panel_TTabSheet.child_window(title="Parcelamento")

            tipo_cobranca = panel_TabParcelamento.child_window(
            class_name="TDBIComboBox", found_index=0
            )

            console.print("Verificando o tipo de cobrança selecionado... \n")
            tipo_selecionado = tipo_cobranca.window_text()
            if "boleto" in tipo_selecionado.lower() or 'carteira' in tipo_selecionado.lower():
                console.print(f"Tipo de cobrança corretamente selecionado {tipo_selecionado}... \n")
            else:
                console.print(f"Tipo de cobrança não foi selecionado corretamente, interagindo com o campo para selecionar o campo corretamente... \n")
                tipo_cobranca.click()
                try:
                    set_combobox("||List", "BANCO DO BRASIL BOLETO")
                except:
                    set_combobox("||List", "CARTEIRA")
        except Exception as e:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Não foi possivel acessar a aba de 'Pagamento', {e}",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )
        try:
            await worker_sleep(2)
            tab_valores = panel_TabPagamento.child_window(title="Valores")
            valores_restantes = tab_valores.child_window(
                class_name="TDBIEditNumber", found_index=1
            )

            valores_informado = tab_valores.child_window(
                class_name="TDBIEditNumber", found_index=2
            )

            valores_informado_text = valores_informado.window_text()
            valores_restantes_text = valores_restantes.window_text()

            valores_informado_text_transform = valores_informado_text.replace('.', '')
            valores_informado_text_transform = valores_informado_text_transform.replace(",",".")

            console.print(f"Pagamento informado valor:{valores_informado_text}... \n")
            if float(valores_informado_text_transform) <= 0.0:
            #if '0,00' in valores_informado_text and len(valores_informado_text) <= 4:
                console.print(f"Pagamento não informado, registrando... \n")
                dt_emissao = nota.get("dataEmissao")
                dt_emissao = datetime.strptime(dt_emissao, "%d/%m/%Y")
                pattern = r"(\d{2}/\d{2}/\d{4})"
                match = re.search(pattern, nota.get("recebimentoFisico"))
                recebimento_fisico = match.group(1) if match else None
                recebimento_fisico = datetime.strptime(recebimento_fisico, "%d/%m/%Y")

                #se a data do aceite no Ahead ultrapassar dois dias após a emissão da nota,  deve-se colocar o vencimento para a mesma data do “Receb. Físico”/Aceite.
                if ((recebimento_fisico >= dt_emissao + timedelta(days=2)) and ("vibra" in nota.get("nomeFornecedor").lower() or "ipiranga" in nota.get("nomeFornecedor").lower() or "raizen" in nota.get("nomeFornecedor").lower() or "charru" in nota.get("nomeFornecedor").lower())):
                    recebimento_fisico = recebimento_fisico.strftime("%d/%m/%Y")
                    console.print(f"Informando a data de vencimento, {recebimento_fisico}... \n")
                    vencimento = panel_TabParcelamento.child_window(
                        class_name="TDBIEditDate"
                    )
                    vencimento.set_edit_text(recebimento_fisico)
                elif "sim dis" in nota.get("nomeFornecedor").lower():
                    vencimento = panel_TabParcelamento.child_window(
                        class_name="TDBIEditDate"
                    )
                    data_vencimento = nota.get("dataVencimento")
                    vencimento.set_edit_text(data_vencimento)
                else:
                    #Senão adicionar 1 dia a emissao
                    dt_emissao = nota.get("dataEmissao")
                    dt_emissao = datetime.strptime(dt_emissao, "%d/%m/%Y")
                    dt_emissao = dt_emissao + timedelta(days=1)
                    dt_emissao = dt_emissao.strftime("%d/%m/%Y")
                    vencimento = panel_TabParcelamento.child_window(
                        class_name="TDBIEditDate"
                    )
                    vencimento.set_edit_text(dt_emissao)

                await worker_sleep(2)
                console.print(f"Inserindo o valor {valores_restantes_text}... \n")
                valor = panel_TabParcelamento.child_window(
                    class_name="TDBIEditNumber", found_index=3
                )
                valor.set_edit_text(valores_restantes_text)
                await worker_sleep(2)
                console.print(f"Adicionando o pagamento... \n")
                btn_add = panel_TabParcelamento.child_window(
                    class_name="TDBIBitBtn", found_index=1
                )
                btn_add.click()

                await worker_sleep(4)
                console.print(f"Verificando se o pagamento foi adicionado com sucesso... \n")
                valores_informado = tab_valores.child_window(
                    class_name="TDBIEditNumber", found_index=2
                )
                valores_informado_text = valores_informado.window_text()
                if '0,00' in valores_informado_text and len(valores_informado_text) == 3:
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=f"Erro ao adicionar o pagamento, valor informado {valores_informado_text}.",
                        status=RpaHistoricoStatusEnum.Falha,
                        tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                    )
                console.print(f"Processo de incluir pagamento realizado com sucesso... \n")
            else:
                data_vencimento = ""
                if "vibra" in nota.get("nomeFornecedor").lower() or "ipiranga" in nota.get("nomeFornecedor").lower() or "raizen" in nota.get("nomeFornecedor").lower() or "charru" in nota.get("nomeFornecedor").lower():
                    dt_emissao = nota.get("dataEmissao")
                    dt_emissao = datetime.strptime(dt_emissao, "%d/%m/%Y")
                    pattern = r"(\d{2}/\d{2}/\d{4})"
                    match = re.search(pattern, nota.get("recebimentoFisico"))
                    recebimento_fisico = match.group(1) if match else None
                    recebimento_fisico = datetime.strptime(recebimento_fisico, "%d/%m/%Y")
                    if recebimento_fisico >= dt_emissao + timedelta(days=2):
                        recebimento_fisico = recebimento_fisico.strftime("%d/%m/%Y")
                        console.print(f"Informando a data de vencimento, {recebimento_fisico}... \n")
                        data_vencimento = recebimento_fisico
                    else:
                        dt_emissao = dt_emissao + timedelta(days=1)
                        dt_emissao = dt_emissao.strftime("%d/%m/%Y")
                        data_vencimento = dt_emissao
                else:
                    data_vencimento = nota.get("dataVencimento")

                await worker_sleep(2)
                console.print(f"Removendo registro de parcelamento do pagamento... \n")
                btn_remove = panel_TabParcelamento.child_window(
                    class_name="TDBIBitBtn", found_index=0
                )
                btn_remove.click()
                await worker_sleep(3)
                confirm_pop_up = await is_window_open_by_class("TMessageForm","TMessageForm")
                if confirm_pop_up["IsOpened"] == True:
                    app_confirm = Application().connect(
                    class_name="TMessageForm"
                    )
                    main_window_confirm = app_confirm["TMessageForm"]

                    btn_yes = main_window_confirm["&Yes"]
                    if btn_yes.exists():
                        try:
                            btn_yes.click()
                            await worker_sleep(3)
                            console.print("O botão Yes de remover parcelamento foi clicado com sucesso.", style="green")
                        except:
                            console.print("Falha ao clicar no botão Yes de faturar.", style="red")
                    else:
                        pyautogui.click(915, 562)
                else:
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=f"Pop de confirmação de remover parcelamento não foi encontrado.",
                        status=RpaHistoricoStatusEnum.Falha,
                        tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                    )
                await worker_sleep(3)
                confirm_pop_up = await is_window_open_by_class("TMessageForm","TMessageForm")
                if confirm_pop_up["IsOpened"] == True:
                        return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=f"Erro ao adicionar remover o parcelamento do pagamento.",
                        status=RpaHistoricoStatusEnum.Falha,
                        tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                        )

                await worker_sleep(2)
                console.print(f"Parcelamento de pagamento excluido, adicionando o novo... \n")
                console.print(f"Inserindo a data de vencimento {data_vencimento} \n")
                vencimento = panel_TabParcelamento.child_window(
                        class_name="TDBIEditDate"
                    )
                vencimento.set_edit_text(data_vencimento)
                await worker_sleep(2)
                app = Application().connect(class_name="TFrmNotaFiscalEntrada")
                main_window = app["TFrmNotaFiscalEntrada"]

                main_window.set_focus()
                panel_TPage = main_window.child_window(class_name="TPage", title="Formulario")
                panel_TTabSheet = panel_TPage.child_window(class_name="TPageControl")

                panel_TabPagamento = panel_TTabSheet.child_window(class_name="TTabSheet")
                panel_TabParcelamento = panel_TTabSheet.child_window(title="Parcelamento")

                tab_valores = panel_TabPagamento.child_window(title="Valores")
                valores_restantes = tab_valores.child_window(
                    class_name="TDBIEditNumber", found_index=1
                )

                valores_informado = tab_valores.child_window(
                    class_name="TDBIEditNumber", found_index=2
                )


                valores_informado_text = valores_informado.window_text()
                valores_restantes_text = valores_restantes.window_text()

                console.print(f"Valor informado: {valores_informado_text} \n")
                console.print(f"Valor restante: {valores_restantes_text} \n")


                console.print(f"Inserindo o valor {valores_restantes_text}... \n")
                valor = panel_TabParcelamento.child_window(
                    class_name="TDBIEditNumber", found_index=3
                )
                valor.set_edit_text(valores_restantes_text)
                await worker_sleep(5)
                console.print(f"Adicionando o pagamento... \n")
                btn_add = panel_TabParcelamento.child_window(
                    class_name="TDBIBitBtn", found_index=1
                )
                btn_add.click()

                await worker_sleep(4)
                console.print(f"Verificando se o pagamento foi adicionado com sucesso... \n")
                valores_informado = tab_valores.child_window(
                    class_name="TDBIEditNumber", found_index=2
                )
                valores_informado_text = valores_informado.window_text()
                if '0,00' in valores_informado_text and len(valores_informado_text) == 3:
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=f"Erro ao adicionar o pagamento, valor informado {valores_informado_text}.",
                        status=RpaHistoricoStatusEnum.Falha,
                        tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                    )
                console.print(f"Processo de incluir pagamento realizado com sucesso... \n")             
        except Exception as e:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Erro ao adicionar o pagamento {e}.",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )


        # Inclui registro
        console.print(f"Incluindo registro...\n")
        try:
            ASSETS_PATH = "assets"
            inserir_registro = pyautogui.locateOnScreen(
                ASSETS_PATH + "\\entrada_notas\\IncluirRegistro.png", confidence=0.8
            )
            pyautogui.click(inserir_registro)
        except Exception as e:
            console.print(
                f"Não foi possivel incluir o registro utilizando reconhecimento de imagem, Error: {e}...\n tentando inserir via posição...\n"
            )
            try:
                retorno = await emsys.incluir_registro(chave_nfe=nota.get("nfe"))
                if retorno.sucesso == True:
                    return RpaRetornoProcessoDTO(
                        sucesso=error_work.sucesso,
                        retorno=error_work.retorno,
                        status=error_work.status,
                    )
            except:
                console.print("A Nota fiscal ainda não foi incluída, continuando o processo...")
        

        await worker_sleep(5)
        console.print(
            "Verificando a existencia de POP-UP de Itens que Ultrapassam a Variação Máxima de Custo ...\n"
        )
        itens_variacao_maxima = await is_window_open_by_class(
            "TFrmTelaSelecao", "TFrmTelaSelecao"
        )
        if itens_variacao_maxima["IsOpened"] == True:
            app = Application().connect(class_name="TFrmTelaSelecao")
            main_window = app["TFrmTelaSelecao"]
            send_keys("%o")
        

        # Verificando se possui pop-up de Warning 
        await worker_sleep(6)
        warning_pop_up = await is_window_open("Warning")
        if warning_pop_up["IsOpened"] == True:
            app = Application().connect(title="Warning")
            main_window = app["Warning"]
            main_window.set_focus()

            console.print(f"Obtendo texto do Warning...\n")
            console.print(f"Tirando print da janela do warning para realização do OCR...\n")

            window_rect = main_window.rectangle()
            screenshot = pyautogui.screenshot(
                region=(
                    window_rect.left,
                    window_rect.top,
                    window_rect.width(),
                    window_rect.height(),
                )
            )
            username = getpass.getuser()
            path_to_png = f"C:\\Users\\{username}\\Downloads\\warning_popup_{nota.get("nfe")}.png"
            screenshot.save(path_to_png)
            console.print(f"Print salvo em {path_to_png}...\n")

            console.print(
                f"Preparando a imagem para maior resolução e assertividade no OCR...\n"
            )
            image = Image.open(path_to_png)
            image = image.convert("L")
            enhancer = ImageEnhance.Contrast(image)
            image = enhancer.enhance(2.0)
            image.save(path_to_png)
            console.print(f"Imagem preparada com sucesso...\n")
            console.print(f"Realizando OCR...\n")
            captured_text = pytesseract.image_to_string(Image.open(path_to_png))
            console.print(
                f"Texto Full capturado {captured_text}...\n"
            )
            os.remove(path_to_png)
            if 'movimento não permitido' in captured_text.lower():
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Filial: {filialEmpresaOrigem} está com o livro fechado ou encerrado, verificar com o setor fiscal",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)]
                )
            elif 'informe o tipo de' in captured_text.lower():
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Mensagem do Warning, Informe o tipo cobraça ",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                )
            else:
                return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Warning não mapeado para seguimento do robo, mensagem: {captured_text}",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                )


        # VERIFICANDO A EXISTENCIA DE ERRO
        erro_pop_up = await is_window_open("Erro")
        if erro_pop_up["IsOpened"] == True:
            await worker_sleep(6)
            error_work = await error_after_xml_imported()
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=error_work.retorno,
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )
        
        await worker_sleep(8)
        nf_chave_acesso = int(nota.get("nfe"))
        status_nf_emsys = await get_status_nf_emsys(nf_chave_acesso)
        if status_nf_emsys.get("status") == "Lançada":
            console.print("\nNota lançada com sucesso, processo finalizado...", style="bold green")
            return RpaRetornoProcessoDTO(
                sucesso=True,
                retorno="Nota Lançada com sucesso!",
                status=RpaHistoricoStatusEnum.Sucesso,
            )
        else:
            console.print("Dupla confirmação lançamento da nota fiscal")
            # Verifica se a info 'Nota fiscal incluida' está na tela
            nf_imported = await check_nota_importada(nota.get("nfe"))
            if nf_imported.sucesso == True:
                await worker_sleep(12)
                console.print("\nVerifica se a nota ja foi lançada...")
                status_nf_emsys = await get_status_nf_emsys(nf_chave_acesso)
                if status_nf_emsys.get("status") == "Lançada":
                    console.print("\nNota lançada com sucesso, processo finalizado...", style="bold green")
                    return RpaRetornoProcessoDTO(
                        sucesso=True,
                        retorno="Nota Lançada com sucesso!",
                        status=RpaHistoricoStatusEnum.Sucesso,
                    )
                else:
                    console.print("Erro ao lançar nota", style="bold red")
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=f"Pop-up nota incluida encontrada, porém nota encontrada como 'já lançada' trazendo as seguintes informações: {nf_imported.retorno} - {error_work}",
                        status=RpaHistoricoStatusEnum.Falha,
                        tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)]
                    )
            else:
                console.print("Erro ao lançar nota", style="bold red")
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Erro ao lançar nota, erro: {nf_imported.retorno}",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                )

    except Exception as ex:
        observacao = f"Erro Processo Entrada de Notas: {str(ex)}"
        logger.error(observacao)
        console.print(observacao, style="bold red")
        
        return RpaRetornoProcessoDTO(
            sucesso=False,
            retorno=observacao,
            status=RpaHistoricoStatusEnum.Falha,
            tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
        )

    finally:
        # Deleta o xml
        await delete_xml(nota["nfe"])



from datetime import datetime
from typing import List, Optional

from pydantic import BaseModel, Field

from worker_automate_hub.models.dto.rpa_sistema_dto import RpaSistemaDTO


class RpaProcessoEntradaDTO(BaseModel):
    datEntradaFila: datetime = Field(..., alias="datEntradaFila")
    configEntrada: dict = Field(..., alias="configEntrada")
    uuidProcesso: str = Field(..., alias="uuidProcesso")
    nomProcesso: str = Field(..., alias="nomProcesso")
    uuidFila: str = Field(..., alias="uuidFila")
    sistemas: List[RpaSistemaDTO] = Field(..., alias="sistemas")
    historico_id: Optional[str] = None
    prioridade: Optional[int] = None

    class Config:
        populate_by_name = True



from mimetypes import guess_type

import aiohttp
from rich.console import Console

from worker_automate_hub.config.settings import load_env_config
from worker_automate_hub.models.dto.rpa_historico_request_dto import RpaHistoricoStatusEnum, RpaRetornoProcessoDTO, RpaTagDTO, RpaTagEnum
from worker_automate_hub.utils.logger import logger

console = Console()


async def send_file_to_datalake(
    directory: str, file: bytes, filename: str, file_extension: str = None
) -> None:
    """
    Envia um arquivo para a datalake.

    Args:
        file (bytes): O conteúdo binário do arquivo.
        filename (str): O nome do arquivo.
        file_extension (str, optional): A extensão do arquivo. Caso não seja
            passada, tenta determinar com base no nome do arquivo.

    Raises:
        aiohttp.ClientResponseError: Caso a API retorne um status de erro.
        Exception: Caso ocorra um erro genérico durante o processo.

    Returns:
        None
    """
    try:
        env_config, _ = load_env_config()

        if not file_extension:
            file_extension = filename.split(".")[-1]

        mime_type, _ = guess_type(filename)
        if not mime_type:
            mime_type = "application/octet-stream"

        body = aiohttp.FormData()
        body.add_field("file", file, filename=filename, content_type=mime_type)
        body.add_field("directoryBucket", directory)

        headers_basic = {"Authorization": f"Basic {env_config['API_AUTHORIZATION']}"}

        async with aiohttp.ClientSession(
            connector=aiohttp.TCPConnector(ssl=False)
        ) as session:
            async with session.post(
                f"{env_config['API_BASE_URL']}/arquivo/send-file-to-datalake",
                data=body,
                headers=headers_basic,
            ) as response:
                response.raise_for_status()

                response_text = await response.text()

                log_msg = f"\nSucesso ao enviar arquivo: {filename}\nResposta da API: {response_text}"
                console.print(log_msg, style="bold green")
                logger.info(log_msg)

    except aiohttp.ClientResponseError as e:
        err_msg = f"Erro na resposta da API: {e.status} - {e.message}\nDetalhes: {await e.response.text()}"
        console.print(f"\n{err_msg}\n", style="bold red")
        logger.error(err_msg)
        return RpaRetornoProcessoDTO(
            sucesso=False,
            retorno=err_msg,
            status=RpaHistoricoStatusEnum.Falha,
            tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
        )

    except Exception as e:
        err_msg = f"Erro ao enviar arquivo: {str(e)}"
        console.print(f"\n{err_msg}\n", style="bold red")
        return RpaRetornoProcessoDTO(
            sucesso=False,
            retorno=err_msg,
            status=RpaHistoricoStatusEnum.Falha,
            tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
        )
